;;----------------------------------------------------------------------
;; Trivial backwards-chaining demo for Jess. Note how each calculation gets
;; done at most once, only if needed.
;; ----------------------------------------------------------------------


;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Turn on backwards chaining for this pattern. IMPORTANT: this must be done
;; BEFORE rules containing this pattern are defined!
(do-backward-chaining factorial)

(deffacts one-value (factorial 5 120))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A rule for which the necesaary LHS facts won't be available; it will trigger
;; backwards chaining. 
(defrule print-factorial-10
  (factorial 10 ?r1)
  =>
  (printout t "The factorial of 10 is " ?r1 crlf))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A rule for which two of the necesaary LHS facts won't be available;
;; it will trigger backwards chaining. NOte that although two rules will require
;; the factorial of 10, it will only be calculated once.
(defrule print-factorial-10-and-12
  (factorial 12 ?r1)
  (factorial 10 ?r2)
  =>
  (printout t "The factorial of 12 is " ?r1
              ", and of 10 is " ?r2 crlf))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; A rule for which some of the needed LHS facts won't be available, but some will.
;; Backwards chaining will only be engaged for the missing ones.
;; 
(defrule print-factorial-5-10-and-12
  "Shows how chaining won't be done if fact already exists"
  (factorial 5 ?r0)
  (factorial 12 ?r1)
  (factorial 10 ?r2)
  =>
  (printout t "The factorial of 12 is " ?r1
              ", and of 10 is " ?r2 ", and of 5 is " ?r0 crlf))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; We don't want to backwards-chain on this, since the number would be too big to
;; calculate, anyway!

(defrule print-factorial-1000
  "Shows how explicit modifer can be used to prevent backwards chaining."
  ?a <- (explicit (factorial 1000 ?r1))
  =>
  (printout t "The factorial of 1000 is " ?r1 crlf))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This is a backwards-chaining-response rule: adding the prefix 'need-' to the name
;; of a chainable deftemplate makes this so.

(defrule do-factorial
  (need-factorial ?x ?)
  =>
  (printout t "( Calculating the factorial of " ?x "... )" crlf)
  (bind ?r 1)
  (bind ?n ?x)
  (while (> ?n 1)
    (bind ?r (* ?r ?n))
    (bind ?n (- ?n 1)))
  (assert (factorial ?x ?r)))

(reset)
(run)


