/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package jessttw;

import java.util.*;

import jess.*;
import ToyTank.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class ReteConnection implements Brain {

    public static final String SENSOR = "SENSOR";

    public static final String TURN = "TURN";
    public static final String MOVE = "MOVE";
    public static final String AIM = "AIM";
    public static final String FIRE = "FIRE";

    final static float radarShortRange = 8f;

    // PI made float instead of double
    final static float PI = (float) Math.PI;


    private Rete rete;
    private Vector facts;

    public ReteConnection(Rete r) {

        rete = r;

        try {
            resetExchangeVariables(rete);
        }
        catch (jess.JessException e) {
            throw new RuntimeException("Internal error: " + e);
        }
    }

    public void sensor(TankSensor tankSensor) {

        facts = new Vector();

        // A sensor command has been received
        facts.addElement(SENSOR);

            // Turret view, tanks only

        if (tankSensor.turretRadar(PI / -18f, PI / +18f, 255))
            facts.addElement("TankCenterTurret");

        if (tankSensor.turretRadar(PI / -3f, PI / -18f, 255))
            facts.addElement("TankLeftTurret");

        if (tankSensor.turretRadar(PI / +18f, PI / +3f, 255))
            facts.addElement("TankRightTurret");


            // Close tanks

        if (tankSensor.tankRadar((2 * PI) / 3f, (4 * PI) / 3f,
                                 radarShortRange))
            facts.addElement("TankBackSide");

        if (tankSensor.tankRadar((2 * PI) / -3f, PI / -3f, radarShortRange))
            facts.addElement("TankLeftSide");

        if (tankSensor.tankRadar(PI / 3f, (2 * PI) / 3f, radarShortRange))
            facts.addElement("TankRightSide");

            // Close obstacles

        if (tankSensor.obstacleSensor((2 * PI) / 3f, (4 * PI) / 3f, 0))
            facts.addElement("ObstacleBackSide");

        if (tankSensor.obstacleSensor(PI / -3f, PI / 3f, 0))
            facts.addElement("ObstacleFrontSide");

        if (tankSensor.obstacleSensor((2 * PI) / -3f, PI / -3f, 0))
            facts.addElement("ObstacleLeftSide");

        if (tankSensor.obstacleSensor(PI / 3f, (2 * PI) / 3f, 0))
            facts.addElement("ObstacleRightSide");


            // Collisions

        if (tankSensor.collisionSensor((2 * PI) / 3f, (4 * PI) / 3f))
            facts.addElement("CollisionBackSide");

        if (tankSensor.collisionSensor(PI / -3f, PI / 3f))
            facts.addElement("CollisionFrontSide");

        if (tankSensor.collisionSensor((2 * PI) / -3f, PI / -3f))
            facts.addElement("CollisionLeftSide");

        if (tankSensor.collisionSensor(PI / 3f, (2 * PI) / 3f))
            facts.addElement("CollisionRightSide");

    }

    public void action(TankAction tankAction) {

        Enumeration enum = null;
        Value value = null;

        try {
            resetExchangeVariables(rete);

            // Assert facts
            enum = facts.elements();

            while (enum.hasMoreElements()) {
                rete.assertString("(" + (String) enum.nextElement() + ")");
            }

            // Run
            rete.run();

            // Extract variables and set action

            value = rete.fetch(TURN);

            if (value != null && value.type() == RU.FLOAT) {
                tankAction.turn((float) value.floatValue(rete.getGlobalContext()));
            }
            else {
                throw new RuntimeException("Invalid value in " + TURN
                                           + " variable");
            }

            value = rete.fetch(MOVE);

            if (value != null && value.type() == RU.FLOAT) {
                tankAction.move((float) value.floatValue(rete.getGlobalContext()));
            }
            else {
                throw new RuntimeException("Invalid value in " + MOVE
                                           + " variable");
            }

            value = rete.fetch(AIM);

            if (value != null && value.type() == RU.FLOAT) {
                tankAction.aim((float) value.floatValue(rete.getGlobalContext()));
            }
            else {
                throw new RuntimeException("Invalid value in " + AIM
                                           + " variable");
            }

            value = rete.fetch(FIRE);

            if (value != null && value.type() == RU.INTEGER) {
                int v = value.intValue(rete.getGlobalContext());

                if (v > 0) {
                    tankAction.fire();
                }
            }
            else {
                throw new RuntimeException("Invalid value in " + AIM
                                           + " variable");
            }

            // Retract facts
            enum = facts.elements();

            while (enum.hasMoreElements()) {
                rete.retractString("(" + (String) enum.nextElement() + ")");
            }
        }
        catch (jess.JessException e) {
            throw new RuntimeException("Internal error: " + e);
        }

    }

    public void restart() {

        try {

            // Reset rete
            rete.reset();
        }
        catch (jess.JessException e) {
            throw new RuntimeException("Internal error: " + e);
        }
    }

    // Just to implement the interface

    public void stop() {}

    public void close() {}

    public void close(int finalGap) {}


    private void resetExchangeVariables(Rete rete) throws JessException {

        // Initialize exchange variables
        rete.store(TURN, new Value(0d, RU.FLOAT));
        rete.store(MOVE, new Value(0d, RU.FLOAT));
        rete.store(AIM, new Value(0d, RU.FLOAT));
        rete.store(FIRE, new Value(0, RU.INTEGER));
    }

}

