/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/**
 * Title:        <p>
 * Description:  <p>
 * Copyright:    Copyright (c) <p>
 * Company:      <p>
 * @author
 * @version 1.0
 */

package npttw;

import java.io.*;
import java.net.*;
import java.util.*;

import ec.*;
import ec.gp.koza.KozaFitness;
import ec.util.*;

import evision.ec.np.*;


import ToyTank.*;


public class NPTTW extends NPProblem implements Brain {

    public static final String P_HOST = "hostname";
    public static final String P_TTWPORT = "ttwport";
    public static final String P_RCPORT = "rcport";

    public static final String TANK_NAME = "NPTank";

    public static final String C_RESET = "RESET";
    public static final String C_START = "START";

    public static final int NO_HIT_MALUS = 100;

    public String hostname;
    public int ttwport;
    public int rcport;

    public NPTTWData input;

    public float move;
    public float turn;
    public float aim;
    public boolean fire;

    protected long gameCounter = 0;

    // Avoid serialization of dynamic data structures
    protected transient Socket rcSocket;
    protected transient BufferedReader rcInStream;
    protected transient PrintStream rcOutStream;

    protected transient TankSensor tankSensor;

    public void setup(final EvolutionState state, final Parameter base) {

        // very important, remember this
        super.setup(state, base);

        // which TTWP host?
        hostname = state.parameters.getStringWithDefault(base.push(P_HOST),
                "localhost");

        // which TTWP port?
        ttwport = state.parameters.getInt(base.push(P_TTWPORT), null, 1);

        // which remote control port?
        rcport = state.parameters.getInt(base.push(P_RCPORT), null, 1);

        // Pretty useless, for now
        input = new NPTTWData();

        ensureRemoteControl(state);
    }

    public void evaluate(final EvolutionState state, final Individual[] ind,
                         final int threadnum) {

        // Will be called after checkpointing
        ensureRemoteControl(state);

        for (int i = 0; i < ind.length; i++) {

            // The fitness function adopted is somewhat random  (i.e. due to
            // random initial tank positioning). Hopefully, the re-evaluation
            // of individuals should mitigate these random effects.
            // if (!ind[i].evaluated)
            try {
                Socket gameSocket = null;
                BufferedReader gameInStream = null;
                PrintStream gameOutStream = null;

                // Reset game
                gameCounter++;

                state.output.message("");
                state.output.message("Evaluation game n. " + gameCounter);

                state.output.message("Resetting game ...");
                rcOutStream.println(RemoteControl.RESET);
                state.output.message(rcInStream.readLine());

                // Register tank
                gameSocket = new Socket(hostname, ttwport);
                gameInStream =
                    new BufferedReader(new InputStreamReader(gameSocket.getInputStream()));
                gameOutStream =
                    new PrintStream(new BufferedOutputStream(gameSocket.getOutputStream(), 1024),
                                    false);

                TTWProtocol protocol = new TTWProtocol(gameInStream,
                                                       gameOutStream);

                // Paint NP tanks with bright yellow
                protocol.writeDescription(TANK_NAME, 255, 255, 128);

                // Start game
                state.output.message("Starting game ...");
                rcOutStream.println(RemoteControl.START);
                state.output.message(rcInStream.readLine());

                // Reset graph
                ((NPIndividual) ind[i]).graphs[0].resetGraph(state,
                        threadnum);

                int malus = 0;
                boolean hit = false;

                int energy = 0;
                int previousEnergy = 0;

                // Play the game, continue until the server says it's over
                while (true) {

                    String inputLine = protocol.readCommand();

                    if (inputLine.startsWith(TTWProtocol.SENSOR)) {

                        // Extract sensor and evaluate
                        tankSensor = protocol.readSensor(inputLine);

                        energy = tankSensor.energy();

                        if (previousEnergy != 0 && energy > previousEnergy) {
                            hit = true;
                        }

                        previousEnergy = energy;

                        // Reset action
                        turn = 0f;
                        move = 0f;
                        aim = 0f;
                        fire = false;

                        // Evaluate individual
                        ((NPIndividual) ind[i]).graphs[0].eval(state,
                                                               threadnum,
                                                               input, null,
                                                               ((NPIndividual) ind[i]),
                                                               this);

                        // Write action
                        protocol.writeAction(this);
                    }
                    else if (inputLine.startsWith(TTWProtocol.RESTART)) {

                        // Reset individual
                        if (!hit) {
                            malus += NO_HIT_MALUS;
                        }

                        hit = false;

                        ((NPIndividual) ind[i]).graphs[0].resetGraph(state,
                                threadnum);
                    }
                    else if (inputLine.startsWith(TTWProtocol.CLOSE)) {

                        // Get the final score
                        int score = protocol.readFinalGap(inputLine);

                        if (!hit) {
                            malus += NO_HIT_MALUS;
                        }

                        score += malus;

                        state.output.message("Game over (score: " + score
                                             + ")");

                        // The fitness better be KozaFitness!
                        KozaFitness f = ((KozaFitness) ind[i].fitness);

                        f.setFitness(state, (float) score);

                        ind[i].evaluated = true;

                        // Quit game
                        protocol.close();
                        gameSocket.close();

                        break;
                    }
                }

            }
            catch (UnknownHostException e) {
                state.output.fatal("Don't know about host: " + hostname);
            }
            catch (IOException e) {
                state.output.fatal("IO error: " + e);
            }
            catch (Exception e) {
                state.output.fatal("Error: " + e);
            }
        }
    }

    // This will be called in NPNodes
    public TankSensor getTankSensor() {
        return tankSensor;
    }

    /* Brain interface - just to ease the integration with TTWProtocol */

    // This method only will actually be used
    public void action(TankAction t) {
        t.turn(turn);
        t.move(move);
        t.aim(aim);

        if (fire)
            t.fire();
    }

    public void sensor(TankSensor t) {
        throw new RuntimeException("Should not be called");
    }

    public void restart() {
        throw new RuntimeException("Should not be called");
    }

    public void stop() {
        throw new RuntimeException("Should not be called");
    }

    public void close() {
        throw new RuntimeException("Should not be called");
    }

    public void close(int finalGap) {
        throw new RuntimeException("Should not be called");
    }


    protected void ensureRemoteControl(final EvolutionState state) {
        if (rcSocket != null)
            return;

            // open remote control and get the welcome msg

        try {
            rcSocket = new Socket(hostname, rcport);
            rcInStream =
                new BufferedReader(new InputStreamReader(rcSocket.getInputStream()));
            rcOutStream =
                new PrintStream(new BufferedOutputStream(rcSocket.getOutputStream(), 1024),
                                true);

            // open remote control and get the welcome msg
            state.output.message("Connecting to server remote control:");
            state.output.message(rcInStream.readLine());
        }
        catch (Exception e) {
            state.output.fatal("Could not connect to server: " + e);
        }
    }

}

