/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * pmarco@ipvvis.unipv.it
 */

package ToyTank;

import java.net.*;
import java.io.*;


public class RemoteControl implements Runnable {
    public static final String BANNER =
        "ToyTankWorld Server - Remote Control";

    public static final String START = "START";
    public static final String RESET = "RESET";
    public static final String QUIT = "QUIT";

    public static final String OK = "200 OK";
    public static final String ERROR = "500 ERROR";

    public static int port = 2002;

    private World world;
    private ServerSocket serverSocket;
    private Socket clientSocket;

    private boolean running = true;

    // Create a server for handling incoming connections
    RemoteControl(World w) {
        world = w;
    }

    // Create a server for handling incoming connections
    RemoteControl(World w, Socket cs) {
        world = w;
        clientSocket = cs;
    }

    /*
     * public interface Runnable
     * (Two kinds of run: server, client setup, client exchange)
     */

    public void run() {

        if (clientSocket == null) {

            // Main thread - handle incoming connections
            ServerSocket serverSocket = null;

            // Start connection port
            try {
                serverSocket = new ServerSocket(port);
            }
            catch (IOException e) {
                System.err.println("Could not listen on port: "
                                   + port + ", " + e.getMessage());
                System.exit(1);
            }

            // Accept new connections
            while (running) {
                try {
                    clientSocket = serverSocket.accept();
                }
                catch (IOException e) {
                    System.err.println("Accept failed: " + e.getMessage());

                    continue;
                }

                // Spawn a new connection
                (new Thread(new RemoteControl(world, clientSocket))).start();

                world.transcript.append("New remote controller connected.\n");
            }

            // Close connections
            try {
                serverSocket.close();
            }
            catch (IOException e) {
                System.err.println("Could not close server socket."
                                   + e.getMessage());
            }
        }
        else {

            // Receive and execute remote commands
            try {

                // Set up protocol
                BufferedReader inStream =
                    new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
                PrintStream outStream =
                    new PrintStream(new BufferedOutputStream(clientSocket.getOutputStream(), 1024),
                                    true);

                outStream.println(BANNER);

                // Wait for remote commands
                while (running) {
                    String command = null;

                    command = inStream.readLine();

                    if (command.equals(START)) {

                        // First do the action and then acknowledge:
                        // a subtke race condition may ensue otherwise
                        world.startAnimation();
                        outStream.println(OK);
                    }
                    else if (command.equals(RESET)) {

                        // First do the action and then acknowledge:
                        // a subtke race condition may ensue otherwise
                        world.resetAnimation();
                        outStream.println(OK);
                    }
                    else if (command.equals(QUIT)) {
                        outStream.println(OK);

                        clientSocket.close();
                        break;
                    }
                    else {
                        outStream.println(ERROR);
                        System.err.println("RC - Unknown command: " + command);

                        clientSocket.close();
                        break;
                    }
                }
            }
            catch (IOException e) {

                // At least try to close it
                try {
                    clientSocket.close();
                    e.printStackTrace();
                }
                catch (IOException e2) {
                    e2.printStackTrace();
                }
            }
        }
    }
}

