/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * pmarco@ipvvis.unipv.it
 */

package ToyTank;

import java.io.*;
import java.net.*;

public final class TankClient implements Runnable {
    public static Class BRAIN_CLASS = ToyTank.Classifier.class;

    private Brain brain;
    private Socket socket;
    private TTWProtocol protocol;

    public TankClient(TTWProtocol p) {
        protocol = p;

        // Construct a new brain
        if (BRAIN_CLASS == Classifier.class) {
            brain = new Classifier(new ADT.Random());
        }
        else {
            try {
                brain = (Brain) BRAIN_CLASS.newInstance();
            }
            catch (Exception e) {
                throw new RuntimeException("Problem in brain creation: " + e);
            }
        }
    }

    public void run() {
        String command;

        while (true) {
            try {
                command = protocol.readCommand();

                if (protocol.isRestartCommand(command)) {

                    // Ok.
                }
                else if (protocol.isStopCommand(command)) {

                    // Ok.
                }
                else if (protocol.isCloseCommand(command)) {
                    return;
                }
                else if (protocol.isSensorCommand(command)) {
                    brain.sensor(protocol.readSensor(command));
                    protocol.writeAction(brain);
                }
                else
                    throw new RuntimeException("Protocol error");
            }
            catch (Exception e) {
                e.printStackTrace();

                return;
            }
        }
    }

    public static void main(String[] args) {
        Socket socket = null;
        PrintStream outStream = null;
        BufferedReader inStream = null;

        if (args.length < 2 || args.length > 6) {
            System.err.println("Usage: TankClient <hostname> <port> [<name> [<r> <g> <b>]]");

            return;
        }

        // Define brain class
        String className = System.getProperty("brainClass", null);

        if (className != null) {
            try {
                Class clazz = Class.forName(className);

                // Is that of the right type?
                if (!Brain.class.isAssignableFrom(clazz))
                    throw new RuntimeException("Not of Brain type: " + clazz);

                BRAIN_CLASS = clazz;
            }
            catch (Exception e) {
                System.err.println("Error while loading brainClass");
                e.printStackTrace();

                // Leave BRAIN_CLASS as is
            }
        }

        try {
            socket = new Socket(args[0], Integer.decode(args[1]).intValue());
            inStream =
                new BufferedReader(new InputStreamReader(socket.getInputStream()));
            outStream =
                new PrintStream(new BufferedOutputStream(socket.getOutputStream(), 1024),
                                false);
        }
        catch (UnknownHostException e) {
            System.err.println("Don't know about host: " + args[0]);
        }
        catch (IOException e) {
            System.err.println("Couldn't get I/O for the connection to: "
                               + args[0] + " " + args[1]);
        }

        if (socket != null && outStream != null && inStream != null) {
            try {
                TTWProtocol protocol = new TTWProtocol(inStream, outStream);

                // Setup connection
                String name = args.length > 2 ? args[2] : "<Unknown>";

                // A triple 0 means "random color"
                int r = 0;
                int g = 0;
                int b = 0;

                try {
                    if (args.length > 3) {
                        r = Integer.valueOf(args[3]).intValue();

                        if (args.length > 4) {
                            g = Integer.valueOf(args[4]).intValue();

                            if (args.length > 5)
                                b = Integer.valueOf(args[5]).intValue();
                        }
                    }
                }
                catch (NumberFormatException e) {}

                protocol.writeDescription(name, r, g, b);

                System.out.println("Server contacted.");

                TankClient tankClient = new TankClient(protocol);

                tankClient.run();

                protocol.close();
                socket.close();
            }
            catch (UnknownHostException e) {
                System.err.println("Trying to connect to unknown host: " + e);
            }
            catch (IOException e) {
                System.err.println("IOException:  " + e);
            }
        }
    }

}

