/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package ToyTank;

/**
 * Derivata da una versione prodotta da Stefano Casarini
 */

import java.io.*;
import java.net.*;
import java.lang.*;
import java.awt.*;
import java.applet.Applet;


public class TankClientApplet extends Applet implements Runnable {
    static final int ROW = 6;

    public static Class BRAIN_CLASS = ToyTank.Classifier.class;

    public TTWProtocol protocol = null;
    public TTWPSensor sensor = null;
    public Brain brain = null;

    private String parameter;
    private String hostname;
    private String tankname;
    private int port;
    private int red, green, blue;

    private Socket socket = null;

    private Thread thread;

    private Graphics gvis;
    private Graphics g;
    private Image image;

    private int energy = 0;
    private double aim = 0.0;
    private boolean[] collision = new boolean[6];
    private int[] shortRange = new int[12];
    private int[][] longRange = new int[6][6];
    private int maxEnergy = 3000;

    private int[] ampLR = {
        30, 22, 10, 10, 22, 30
    };
    private int[] deltaLR = {
        0, 30, 52, 62, 72, 94
    };


    /**
     * INIT, inizializza l'applet con la decodifica dei parametri
     * e la creazione delle variabili grafiche necessarie per il Doppio Buffering
     */
    public void init() {

        /**
         * Lettura PARAMETRI passati all' APPLET
         */
        parameter = this.getParameter("hostname");
        hostname = new String(parameter);

        parameter = this.getParameter("ttwport");
        port = Integer.parseInt(parameter);

        parameter = this.getParameter("tankname");
        tankname = new String(parameter);

        parameter = this.getParameter("r");
        red = Integer.parseInt(parameter);

        parameter = this.getParameter("g");
        green = Integer.parseInt(parameter);

        parameter = this.getParameter("b");
        blue = Integer.parseInt(parameter);

        // Automatic brain class
        parameter = this.getParameter("brainClass");

        if (parameter != null) {
            try {
                Class clazz = Class.forName(parameter);

                // Is that of the right type?
                if (!Brain.class.isAssignableFrom(clazz))
                    throw new RuntimeException("Not of Brain type: " + clazz);

                BRAIN_CLASS = clazz;
            }
            catch (Exception e) {
                System.err.println("Error while loading brainClass");
                e.printStackTrace();

                // Leave as is
            }
        }

        // Construct a new brain
        if (BRAIN_CLASS == Classifier.class) {
            brain = new Classifier(new ADT.Random());
        }
        else {
            try {
                brain = (Brain) BRAIN_CLASS.newInstance();
            }
            catch (Exception e) {
                throw new RuntimeException("Problem in brain creation: " + e);
            }
        }

        // Connect
        BufferedReader inStream = null;
        PrintStream outStream = null;

        try {
            socket = new Socket(hostname, port);
            inStream =
                new BufferedReader(new InputStreamReader(socket.getInputStream()));
            outStream =
                new PrintStream(new BufferedOutputStream(socket.getOutputStream(), 1024),
                                false);
        }
        catch (UnknownHostException e) {
            System.err.println("Don't know about host: " + hostname);
        }
        catch (IOException e) {
            System.err.println("Couldn't get I/O for the connection to: "
                               + hostname + " " + port);
        }

        if (socket != null && outStream != null && inStream != null) {
            try {
                protocol = new TTWProtocol(inStream, outStream);

                protocol.writeDescription(tankname, red, green, blue);

                System.out.println("Server contacted.");

            }
            catch (UnknownHostException e) {
                System.err.println("Trying to connect to unknown host: " + e);
            }
            catch (IOException e) {
                System.err.println("IOException:  " + e);
            }
        }

        setBackground(Color.white);
        setForeground(Color.black);
    }


    public void destroy() {

        // Close connection
        if (socket != null || protocol != null) {
            try {
                protocol.close();
                socket.close();
            }
            catch (IOException e) {
                System.err.println("IOException:  " + e);
            }
        }
    }

    /**
     * START, Avvio del Thread
     */
    public void start() {

        /**
         * Avvio Thread
         */
        if (thread == null) {
            thread = new Thread(this);

            thread.start();
        }
    }

    /**
     * STOP, fermo il Thread
     */
    public void stop() {

        /**
         * Arresto Thread
         */
        if (thread != null) {
            thread.stop();

            thread = null;
        }
    }

    /**
     * PAINT, disegna gli indicatori di stato del Tank
     */
    public void paint(Graphics gvis) {

        provideBuffer();

        g.setColor(Color.white);
        g.fillRect(0, 0, size().width, size().height);

        g.setColor(Color.black);
        g.drawString("T A N K   C L I E N T ", 5, 20);
        g.drawLine(0, 23, 390, 23);

        g.setColor(Color.red);
        g.drawString("Hostname: " + hostname + " - Port: " + port, 5, 40);
        g.drawString("Name: " + tankname + " - Color (RGB): " + red + ", "
                     + green + ", " + blue, 5, 60);
        g.setColor(new Color(red, green, blue));
        g.drawString("(i.e. this color)", 270, 60);

        if (sensor == null) {

            // Ok, for now
            return;
        }

        // Energy
        g.setColor(Color.black);
        g.drawString("Energy: " + sensor.energy, 40, 90);

        // Turret aim
        float aim = (new Float(sensor.aim)).floatValue();

        g.setColor(Color.black);
        g.drawOval(120, 100, 100, 100);
        g.drawLine((int) (170 + 45 * Math.sin(aim)),
                   (int) (150 - 45 * Math.cos(-aim)), 170, 150);

        g.drawString("Turret", 155, 90);
        g.drawString(new Double(Math.rint(aim * 1000 * 180 / Math.PI)
                                / 1000).toString(), 150, 175);
        g.drawString("90", 223, 155);
        g.drawString("-90", 101, 155);
        g.drawString("180", 160, 212);


        // Collision
        g.setColor(Color.green);
        g.fillArc(245, 95, 110, 110, 30, 60);
        g.fillArc(245, 95, 110, 110, 150, 60);
        g.fillArc(245, 95, 110, 110, 270, 60);

        g.setColor(Color.white);
        g.fillOval(250, 100, 100, 100);

        for (int i = 0; i < 6; i++) {
            if (sensor.collision.charAt(i) == '1') {
                g.setColor(Color.gray);
                g.fillArc(250, 100, 100, 100, 90 - i * 60, -60);
            }
        }

        g.setColor(Color.white);
        g.fillOval(280, 130, 40, 40);

        g.setColor(Color.black);
        g.drawOval(250, 100, 100, 100);
        g.drawOval(280, 130, 40, 40);
        g.drawString("Collision", 280, 90);


        // Short range
        g.setColor(Color.green);
        g.fillArc(35, 215, 110, 110, 0, 30);
        g.fillArc(35, 215, 110, 110, 60, 30);
        g.fillArc(35, 215, 110, 110, 120, 30);
        g.fillArc(35, 215, 110, 110, 180, 30);
        g.fillArc(35, 215, 110, 110, 240, 30);
        g.fillArc(35, 215, 110, 110, 300, 30);

        g.setColor(Color.white);
        g.fillOval(40, 220, 100, 100);

        for (int i = 0; i < 12; i++) {
            if (sensor.shortRange.charAt(i) == '1') {
                g.setColor(Color.blue);
                g.fillArc(40, 220, 100, 100, 90 - i * 30, -30);
            }
            else if (sensor.shortRange.charAt(i) == '2') {
                g.setColor(Color.red);
                g.fillArc(40, 220, 100, 100, 90 - i * 30, -30);
            }
            else if (sensor.shortRange.charAt(i) == '3') {
                g.setColor(Color.magenta);
                g.fillArc(40, 220, 100, 100, 90 - i * 30, -30);
            }
        }

        g.setColor(Color.white);
        g.fillOval(70, 250, 40, 40);

        g.setColor(Color.black);
        g.drawOval(40, 220, 100, 100);
        g.drawOval(70, 250, 40, 40);
        g.drawString("Short Range", 60, 340);


        // Long range
        for (int i = 5; i >= 0; i--) {
            for (int j = 0; j < 6; j++) {
                if (sensor.longRange.charAt((i * ROW) + j) == '1') {
                    g.setColor(Color.blue);
                    g.fillRect((int) (200 + deltaLR[j]),
                               (int) (250 + (5 - i) * 10), ampLR[j], 10);
                }
                else if (sensor.longRange.charAt((i * ROW) + j) == '2') {
                    g.setColor(Color.red);
                    g.fillRect((int) (200 + deltaLR[j]),
                               (int) (250 + (5 - i) * 10), ampLR[j], 10);
                }
                else if (sensor.longRange.charAt((i * ROW) + j) == '3') {
                    g.setColor(Color.magenta);
                    g.fillRect((int) (200 + deltaLR[j]),
                               (int) (250 + (5 - i) * 10), ampLR[j], 10);
                }
            }
        }

        g.setColor(Color.black);
        g.drawRect(200, 250, 124, 60);

        g.drawRect(230, 250, 64, 60);
        g.drawRect(252, 250, 20, 60);
        g.drawLine(262, 250, 262, 310);

        g.drawRect(200, 260, 124, 40);
        g.drawRect(200, 270, 124, 20);
        g.drawLine(200, 280, 324, 280);

        g.drawString("Long Range", 230, 340);

        g.setColor(Color.blue);
        g.fillRect(65, 355, 10, 10);
        g.setColor(Color.black);
        g.drawString("Obstacle", 78, 365);

        g.setColor(Color.red);
        g.fillRect(165, 355, 10, 10);
        g.setColor(Color.black);
        g.drawString("Tank", 178, 365);

        g.setColor(Color.magenta);
        g.fillRect(265, 355, 10, 10);
        g.setColor(Color.black);
        g.drawString("Both", 278, 365);

        // Double buffering
        gvis.drawImage(image, 0, 0, this);
    }


    /**
     * Metodo UPDATE, per l'aggiornamento della grafica
     */
    public void update(Graphics gvis) {
        paint(gvis);
    }


    /**
     * Metodo RUN, codice del programma principale.
     */
    public void run() {

        String command;

        while (true) {
            try {
                command = protocol.readCommand();

                if (protocol.isRestartCommand(command)) {

                    // Ok.
                }
                else if (protocol.isStopCommand(command)) {

                    // Ok.
                }
                else if (protocol.isCloseCommand(command)) {
                    return;
                }
                else if (protocol.isSensorCommand(command)) {
                    sensor = (TTWPSensor) protocol.readSensor(command);

                    // Force redrawing
                    update(getGraphics());

                    brain.sensor(sensor);
                    protocol.writeAction(brain);
                }
                else
                    throw new RuntimeException("Protocol error");
            }
            catch (Exception e) {
                e.printStackTrace();

                return;
            }
        }

    }

    private void provideBuffer() {
        if (g == null) {
            image = createImage(size().width, size().height);
            g = image.getGraphics();
        }
    }

}

