/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * pmarco@ipvvis.unipv.it
 */

package ToyTank;

import java.net.*;
import java.io.*;


public class TankServer implements Runnable, Brain {

    public static int port = 2001;

    private World world;
    private ServerSocket serverSocket;
    private Socket clientSocket;
    private TTWProtocol protocol;
    private TankSensor tankSensor;
    private TankAction tankAction;
    private boolean running = true;
    private boolean accepting = true;
    private static int counter = 0;
    private int thisThread;
    private boolean success = false;

    // Create a server for handling incoming connections
    TankServer(World w) {
        world = w;
    }

    // Create a server for an individual connection
    TankServer(World w, Socket socket) {
        world = w;
        clientSocket = socket;
        thisThread = counter++;
    }

    /*
     * public interface Runnable
     * (Three kinds of run: server, client setup, client exchange)
     */

    public void run() {
        success = false;

        if (clientSocket == null) {

            // Main thread - handle incoming connections
            ServerSocket serverSocket = null;

            // Start connection port
            try {
                serverSocket = new ServerSocket(port);
            }
            catch (IOException e) {
                System.err.println("Could not listen on port: "
                                   + port + ", " + e.getMessage());
                System.exit(1);
            }

            // Accept new connections
            while (running) {
                try {
                    clientSocket = serverSocket.accept();

                    if (!accepting)
                        clientSocket.close();
                }
                catch (IOException e) {
                    System.err.println("Accept failed: " + e.getMessage());

                    continue;
                }

                // Spawn a new connection
                (new Thread(new TankServer(world, clientSocket))).start();
                world.transcript.append("New connection accepted.\n");
            }

            // Close connections
            try {
                serverSocket.close();

                success = true;
            }
            catch (IOException e) {
                System.err.println("Could not close server socket."
                                   + e.getMessage());
            }
        }
        else if (tankAction == null && tankSensor == null) {

            // Set up an individual connection
            try {

                // Set up protocol
                BufferedReader inStream =
                    new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
                PrintStream outStream =
                    new PrintStream(new BufferedOutputStream(clientSocket.getOutputStream(), 1024),
                                    false);

                protocol = new TTWProtocol(inStream, outStream);

                // Try creating a new tank
                Tank tank = new Tank(world, this);

                protocol.readDescription(tank);

                // Ok. Register it
                world.register(tank);
                world.transcript.append("Registered toy tank \""
                                        + tank.getName() + "\" ("
                                        + tank.getColor() + ")\n  from "
                                        + clientSocket.getInetAddress()
                                        + "\n");

                success = true;
            }
            catch (IOException e) {

                // At least try to close it
                try {
                    protocol.close();
                    clientSocket.close();
                    e.printStackTrace();
                }
                catch (IOException e2) {
                    e2.printStackTrace();
                }
            }
        }
        else {

            // Perform an exchange with client
            try {

                // Write sensor and read action
                protocol.writeSensor((TTWPSensor) tankSensor);
                protocol.readAction(tankAction);

                success = true;
            }
            catch (IOException e) {

                // At least try to close it
                try {
                    e.printStackTrace();
                    protocol.close();
                    clientSocket.close();
                }
                catch (IOException e2) {
                    e.printStackTrace();
                }
            }
            finally {
                tankSensor = null;
                tankAction = null;
            }
        }
    }

    public synchronized void sensor(TankSensor t) {

        tankSensor = t;

        // Do nothing, wait for action request
    }

    public synchronized void action(TankAction t) {
        tankAction = t;

        // Do not fork a new thread, yet
        // (new Thread(this)).start();
        run();

        if (!successful())
            throw new RuntimeException("Operation could not complete.");
    }

    public synchronized void restart() {
        try {
            protocol.writeRestart();
        }
        catch (IOException e) {

            // At least try to close it
            try {
                protocol.close();
                clientSocket.close();
                e.printStackTrace();
            }
            catch (IOException e2) {
                e2.printStackTrace();
            }
        }
    }

    public synchronized void stop() {
        try {
            protocol.writeStop();
        }
        catch (IOException e) {

            // At least try to close it
            try {
                protocol.close();
                clientSocket.close();
                e.printStackTrace();
            }
            catch (IOException e2) {
                e2.printStackTrace();
            }
        }
    }

    public synchronized void close() {
        close(0);
    }

    public synchronized void close(int finalGap) {
        try {
            protocol.writeClose(finalGap);
        }
        catch (IOException e) {

            // Useless - we're about to shut it down
        }

        try {
            if (serverSocket != null)
                serverSocket.close();
            else {
                protocol.close();
                clientSocket.close();
            }
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }


    /*
     * Other public methods
     */

    public void setAccepting(boolean a) {
        accepting = a;
    }

    public boolean isDone() {
        if (clientSocket == null)
            throw new RuntimeException("This is the main server");

        return tankSensor == null && tankAction == null;
    }

    public boolean successful() {
        return success;
    }

}

