/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * AI Tank World
 *
 * Applet that controls simulation.
 *
 * Author  Matthew Caryl
 * Created 30.10.96
 *
 * Although under copywrite to the author (Matthew Caryl)
 * this code can be copied and modified for non-commercial
 * purposes as long as any derivatives contain this condition.
 *
 * Modified: pmarco@ipvvis.unipv.it
 */

package ToyTank;

import java.awt.*;
import java.applet.Applet;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.lang.Math;
import java.lang.System;
import ADT.Queue;
import ADT.QueueEmpty;
import ADT.QueueItemAbsent;
import ADT.Random;
import ADT.Physical;
import ADT.BSPT;
import ADT.Model;


public final class World extends Applet implements Runnable {
    public static int ROUNDS = 3;
    public static Class BRAIN_CLASS = ToyTank.Classifier.class;

    // Maths interace
    private static final float PI = (float) Math.PI;
    private static final float TWOPI = (float) Math.PI * 2f;

    private static final float atan(float a) {
        return (float) Math.atan(a);
    }

    private static final int sqr(int n) {
        return n * n;
    }


    // Private world interface
    public static final Color backgroundColor = Color.black;
    public static final Color outlineColor = Color.white;
    public WorldTranscript transcript;

    private TankServer tankServer;
    private RemoteControl remoteControl;

    private Thread animationThread;
    private Control control;
    private Graphics graphics;
    private Random random = new Random();
    private long initial_seed;

    // General parameters
    private int width;
    private int height;
    private int rounds;
    private int delay;
    private int tanks;

    // Game parameters
    private int lifespan;
    private int shellcost;
    private int shellspeed;
    private int shelldamage;
    private int damagetransfer;
    private int turretspeed;
    private int tankspeed;

    private BSPT bspt;
    private Polygon[] polygons;
    private int round = 1;
    private Queue currentPhysicals;
    private Queue newPhysicals;
    private Queue oldPhysicals;
    private Queue oldTanks;
    private Physical[] physicals = new Physical[0];
    private int tickCounter = 0;

    //
    // Public applet interface
    //

    /*
     * Recognized applet parameters:
     * width (int)
     * height (int)
     * [rounds (int)] // rounds in tournament, default 3
     * [delay (int)] // msec, default 2000
     * [tanks (int)] // automatic opponent, default 5
     * worldDescription (String)
     *
     * [lifespan (int)]
     * [shellcost (int)]
     * [shellspeed (int)]
     * [shelldamage (int)]
     * [damagetransfer (int)]
     * [turretspeed (int)]
     * [tankspeed (int)]
     *
     * [brainClass (String)] // automatic brain, default ToyTank.Classifier
     *
     * [debug (boolean)]
     */

    public void init() {
        String parameterValue;

        initial_seed = random.seed();

        setBackground(backgroundColor);

        /* General parameters */

        // Read in width, height, and block size parameters.
        parameterValue = this.getParameter("width");

        try {
            width = Integer.parseInt(parameterValue);

            if (width < 1)
                throw new NumberFormatException("Width must be postive");
        }
        catch (NumberFormatException e) {
            width = 512;
        }

        parameterValue = this.getParameter("height");

        try {
            height = Integer.parseInt(parameterValue);

            if (height < 1)
                throw new NumberFormatException("Height must be postive");
        }
        catch (NumberFormatException e) {
            height = 512;
        }

        // Read in number of tanks in the world
        parameterValue = this.getParameter("tankNumber");

        try {
            tanks = Integer.parseInt(parameterValue);

            if (tanks < 0)
                throw new NumberFormatException("Tank number must be positive");
        }
        catch (NumberFormatException e) {
            tanks = 5;
        }

        // Read in number of rounds
        parameterValue = this.getParameter("rounds");

        try {
            rounds = Integer.parseInt(parameterValue);

            if (rounds < 1)
                throw new NumberFormatException("Rounds number must be postive");

            ROUNDS = rounds;
        }
        catch (NumberFormatException e) {

            // Leave as is
        }

        // Read in delay between rounds
        parameterValue = this.getParameter("delay");

        try {
            delay = Integer.parseInt(parameterValue);

            if (delay < 0)
                throw new NumberFormatException("Delay must be postive");
        }
        catch (NumberFormatException e) {
            delay = 2000;
        }


        /* Game parameters */

        // Read in lifespan
        parameterValue = this.getParameter("lifespan");

        try {
            lifespan = Integer.parseInt(parameterValue);

            if (lifespan < 1)
                throw new NumberFormatException("lifespan must be postive");

            Tank.STARTTIME = lifespan;
        }
        catch (NumberFormatException e) {

            // Leave as is
        }

        // Read in shellcost
        parameterValue = this.getParameter("shellcost");

        try {
            shellcost = Integer.parseInt(parameterValue);

            if (shellcost < 1)
                throw new NumberFormatException("shellcost must be postive");

            Tank.FIRETIME = shellcost;
        }
        catch (NumberFormatException e) {

            // Leave as is
        }

        // Read in shellspeed
        parameterValue = this.getParameter("shellspeed");

        try {
            shellspeed = Integer.parseInt(parameterValue);

            if (shellspeed < 1)
                throw new NumberFormatException("shellspeed must be postive");

            Tank.SHELLSPEED = shellspeed;
        }
        catch (NumberFormatException e) {

            // Leave as is
        }

        // Read in shelldamage
        parameterValue = this.getParameter("shelldamage");

        try {
            shelldamage = Integer.parseInt(parameterValue);

            if (shelldamage < 1)
                throw new NumberFormatException("shelldamage must be postive");

            Tank.TAKETIME = shelldamage;
        }
        catch (NumberFormatException e) {

            // Leave as is
        }

        // Read in damagetransfer
        parameterValue = this.getParameter("damagetransfer");

        try {
            damagetransfer = Integer.parseInt(parameterValue);

            if (damagetransfer < 1)
                throw new NumberFormatException("damagetransfer must be postive");

            Tank.GAINTIME = damagetransfer;
        }
        catch (NumberFormatException e) {

            // Leave as is
        }

        // Read in turretspeed
        parameterValue = this.getParameter("turretspeed");

        try {
            turretspeed = Integer.parseInt(parameterValue);

            if (turretspeed < 1)
                throw new NumberFormatException("turretspeed must be postive");

            Tank.TURRETSPEED = turretspeed;
        }
        catch (NumberFormatException e) {

            // Leave as is
        }


        // Read in tankspeed
        parameterValue = this.getParameter("tankspeed");

        try {
            tankspeed = Integer.parseInt(parameterValue);

            if (tankspeed < 1)
                throw new NumberFormatException("tankspeed must be postive");

            Tank.TANKSPEED = tankspeed;
        }
        catch (NumberFormatException e) {

            // Leave as is
        }


        // Read in ttwport
        parameterValue = this.getParameter("ttwport");

        try {
            int ttwport = Integer.parseInt(parameterValue);

            if (ttwport < 1)
                throw new NumberFormatException("ttwport must be postive");

            TankServer.port = ttwport;
        }
        catch (NumberFormatException e) {

            // Leave as is
        }

        // Read in rcwport
        parameterValue = this.getParameter("rcport");

        try {
            int rcport = Integer.parseInt(parameterValue);

            if (rcport < 1)
                throw new NumberFormatException("rcport must be postive");

            RemoteControl.port = rcport;
        }
        catch (NumberFormatException e) {

            // Leave as is
        }

        // Automatic brain class
        parameterValue = this.getParameter("brainClass");

        if (parameterValue != null) {
            try {
                Class clazz = Class.forName(parameterValue);

                // Is that of the right type?
                if (!Brain.class.isAssignableFrom(clazz))
                    throw new RuntimeException("Not of Brain type: " + clazz);

                BRAIN_CLASS = clazz;
            }
            catch (Exception e) {
                System.err.println("Error while loading brainClass");
                e.printStackTrace();

                // Leave as is
            }
        }

        // Download world description
        downloadWorld(this.getParameter("worldDescription"));

        if (this.getParameter("debug").equals("TRUE"))
            TTWProtocol.setDebug(true);

            // Save graphics context for later
            // Note: this seems to reduce garbage collection problems

        graphics = this.getGraphics();

        // Open transcript
        transcript = new WorldTranscript(this);

        transcript.setTitle("Transcript");
        transcript.pack();
        transcript.show();

        reset();

    }

    // start again from round 1
    public void reset() {

        // Make first tanks
        currentPhysicals = new Queue();
        newPhysicals = new Queue();
        oldPhysicals = new Queue();
        oldTanks = new Queue();

        Tank t;

        for (int i = tanks; i > 0; i--) {
            t = new Tank(this);

            t.setName("Random " + i);
            currentPhysicals.enqueue(placeTank(t));
        }

        round = 1;

        if (tankServer == null) {
            tankServer = new TankServer(this);

            (new Thread(tankServer)).start();
            transcript.append("Connection server started.\n");
        }
        else {
            tankServer.setAccepting(true);
            transcript.append("Connection server restarted.\n");
        }

        if (remoteControl == null) {
            remoteControl = new RemoteControl(this);

            (new Thread(remoteControl)).start();
            transcript.append("Remote control server started.\n");
        }
    }

    // resume thread if already started
    public void start() {

        if (animationThread != null && animationThread.isInterrupted()) {
            animationThread = new Thread(this);

            animationThread.start();
        }
    }

    // Start the real show
    public synchronized void startAnimation() {

        // Save graphics context for later
        // Note: this seems to reduce garbage collection problems
        graphics = this.getGraphics();

        // Stop accepting connections
        tankServer.setAccepting(false);
        transcript.append("Starting animation.\n");

        animationThread = new Thread(this);

        animationThread.start();
    }

    // Stop and reset the show
    public synchronized void resetAnimation() {

        transcript.append("\n\nRe-starting server.\n\n");

        // Stop animation
        stop();

        // Close tanks & connections (if any)
        close();

        // Make a gc (you never know)
        Runtime.getRuntime().gc();

        // Set it up again
        reset();
    }

    // interrupt rather than suspend
    public void stop() {
        if (animationThread != null && animationThread.isAlive())
            animationThread.interrupt();
    }

    // start animation and remove control panel
    // Only called by control panel
    public void restart() {
        start();

        control = null;
    }

    // main combat round cycle
    public void run() {
        for (int i = 1; i <= ROUNDS; i++) {
            round(i);
            delay(delay);
            repaint();
            combat();

            if (i < ROUNDS) {
                reload();
            }
            else {
                close();
            }
        }
    }

    // download polygon description of world
    private void downloadWorld(String worldString) {
        try {
            InputStream is = new URL(getCodeBase(), worldString).openStream();

            polygons = BSPT.parsePolygons(is, width(), height());
            bspt = new BSPT(polygons);
        }
        catch (IOException e) {
            int[] wallx = {
                1, 1, width() - 2, width() - 2, 1
            };
            int[] wally = {
                1, height() - 2, height() - 2, 1, 1
            };

            polygons = new Polygon[1];
            polygons[0] = new Polygon(wallx, wally, 5);
            bspt = new BSPT(polygons);
        }
    }

    // place tank without collision with walls or curret objects
    private Tank placeTank(Tank t) {

        // Force one teleport to cope with subsequent re-placements
        t.teleport();

        while (bspt.in(t.x(), t.y(), t.radius())
               || Model.collision(currentPhysicals, t)) {
            t.teleport();
        }

        return t;
    }

    // battle tanks until all are dead
    private void combat() {
        tickCounter = 0;

        try {
            while (!currentPhysicals.empty()) {
                tickCounter++;

                synchronized (currentPhysicals) {
                    int size = currentPhysicals.size();
                    Physical p;

                    // switch to fast array
                    if (physicals.length < size)
                        physicals = new Physical[size * 2];

                        // 2000-06-18 Avoid spinning idle when all tanks are gone

                    boolean done = true;

                    for (int i = size - 1; i >= 0; i--) {
                        physicals[i] = (Physical) currentPhysicals.requeue();

                        if (physicals[i] instanceof Tank)
                            done = false;
                    }

                    // 2000-06-18 carefully erase and break when done
                    if (done) {
                        graphics.setColor(backgroundColor);

                        while (!currentPhysicals.empty()) {
                            p = (Physical) currentPhysicals.dequeue();

                            p.set(p.x(), p.y());
                            p.erase(graphics);
                        }

                        break;
                    }

                    // give a sensor to each physical
                    for (int i = size - 1; i >= 0; i--)
                        physicals[i].sensor();

                    // give a tick to each physical
                    for (int i = size - 1; i >= 0; i--) {
                        p = physicals[i];

                        p.action();

                        // resolve physical-static collision
                        if (bspt.in(p.x(), p.y(), p.radius()))
                            p.collision(true);
                    }

                    // resolve physical-physical collisions
                    if (Model.collision(currentPhysicals, true))
                        while (Model.collision(currentPhysicals,
                                               false)); // do nothing

                        // remove some of the physicals

                    graphics.setColor(backgroundColor);

                    while (!oldPhysicals.empty()) {
                        p = (Physical) oldPhysicals.dequeue();

                        p.erase(graphics);
                    }

                    // paint and redraw
                    for (int i = currentPhysicals.size(); i > 0; i--) {
                        p = (Physical) currentPhysicals.requeue();

                        graphics.setColor(backgroundColor);
                        p.erase(graphics);

                        p.paint(graphics);
                    }

                    // add some of the physicals
                    while (!newPhysicals.empty()) {
                        p = (Physical) newPhysicals.dequeue();

                        currentPhysicals.enqueue(p);
                        p.paint(graphics);
                    }
                }
            }
        }
        catch (QueueEmpty e) {
            System.err.println("report to programmer - combat");
        }
    }

    // just reload tanks
    private void reload() {
        updateStatistics();

        try {
            int size = oldTanks.size();
            Tank t;

            // Breed
            for (int i = 0; i < size; i++) {
                t = (Tank) oldTanks.dequeue();
                t = t.breed();
                t = placeTank(t);

                currentPhysicals.enqueue(t);
            }
        }
        catch (QueueEmpty e) {
            System.err.println("report to programmer - breed");
        }
    }

    private void close() {
        updateStatistics();
        showStatistics();

        try {
            int size = oldTanks.size();
            Tank t;

            for (int i = 0; i < size; i++) {
                t = (Tank) oldTanks.dequeue();

                t.close();
            }
        }
        catch (QueueEmpty e) {
            System.err.println("report to programmer - close");
        }
    }

    private void updateStatistics() {
        try {
            int size = oldTanks.size();
            Tank t;

            // Update statstics - find pivot
            int bestSurvival = 0;

            for (int i = 0; i < size; i++) {
                t = (Tank) oldTanks.requeue();

                if (t.getSurvival() > bestSurvival)
                    bestSurvival = t.getSurvival();
            }

            // Update statistics
            for (int i = 0; i < size; i++) {
                t = (Tank) oldTanks.requeue();

                t.updateGap(bestSurvival);
            }
        }
        catch (QueueEmpty e) {
            System.err.println("report to programmer - update statistics");
        }
    }

    // display round number in center of applet
    private void round(int round) {
        graphics.setColor(backgroundColor);
        graphics.fillRect(0, 0, width(), height());
        graphics.setColor(outlineColor);

        String message = "Round " + round;
        int messageWidth =
            graphics.getFontMetrics(graphics.getFont()).stringWidth(message);
        int messageHeight =
            graphics.getFontMetrics(graphics.getFont()).getHeight();

        graphics.drawString(message, width() / 2 - messageWidth / 2,
                            height() / 2 - messageHeight / 2);

        transcript.append(message + "\n");
    }

    private void showStatistics() {
        try {
            int size = oldTanks.size();
            Tank[] tanks = new Tank[size];

            if (size > 0)
                transcript.append("\n* Tournament Results *\n");

            for (int i = 0; i < size; i++) {
                tanks[i] = (Tank) oldTanks.requeue();

            }

            quickSort(tanks, 0, tanks.length - 1);

            for (int i = 0; i < size; i++) {
                transcript.append("" + (i + 1) + " : Tank " + tanks[i].getName()
                                  + " - final gap: " + tanks[i].getGap()
                                  + "\n");
            }
        }
        catch (QueueEmpty e) {
            System.err.println("report to programmer - update statistics");
        }
    }

    private void swap(Tank[] tanks, int x, int y) {
        Tank swap = tanks[x];

        tanks[x] = tanks[y];
        tanks[y] = swap;
    }

    private void quickSort(Tank[] tanks, int left, int right) {
        if (left < right) {
            int h = (int) ((right - left) / 2) + left;
            int l, r;
            int gap = tanks[left].getGap();

            for (l = left + 1, r = right; l <= r; )
                if (tanks[l].getGap() < gap)
                    l++;
                else if (tanks[r].getGap() >= gap)
                    r--;
                else {
                    swap(tanks, l, r);

                    l++;
                    r--;
                }

            swap(tanks, left, l - 1);
            quickSort(tanks, left, l - 2);
            quickSort(tanks, l, right);
        }
    }

    // sleep for a brief while
    private void delay(int delay) {
        try {
            Thread.sleep(delay);
        }
        catch (InterruptedException e) {

            // do nothing
        }
    }

    // redisplay landscape
    public void repaint() {
        paint(graphics);
    }

    // display landscape
    public void paint(Graphics g) {
        g.setColor(backgroundColor);
        g.fillRect(0, 0, width(), height());
        g.setColor(outlineColor);
        bspt.draw(g);
    }

    // bring control panel up
    public boolean mouseDown(Event e, int x, int y) {

        if (control == null) {
            stop();

            /*
             * Control panel disabled: use parameters instead
             * control = new Control(this, initial_seed);
             */
        }

        return true;
    }

    public String[][] getParameterInfo() {
        String[][] info = {
             {
                "width ", "positive integer ", "width of world in pixels"
            }, {
                "height ", "positive integer ", "height of world in pixels"
            }, {
                "tankNumber ", "positive integer ", "number of tanks"
            }, {
                "worldDescription", "polygon file name",
                "description of landscape"
            }
        };

        return info;
    }

    // is first angle less than second
    private boolean less(float first, float second) {
        if (first < second)
            return first + PI > second;
        else
            return first > second + PI;
    }

    // ensure angle is between [0..TWOPI]
    private float normalise(float angle) {
        if (angle < 0)
            return angle + TWOPI;
        else if (angle > TWOPI)
            return angle - TWOPI;
        else
            return angle;
    }

    //
    // Public world interface
    //

    public int width() {
        return width;
    }

    public int height() {
        return height;
    }

    public Random random() {
        return random;
    }

    // polygon description of landscape
    public Polygon[] polygons() {
        return polygons;
    }

    public void register(Tank tank) {
        currentPhysicals.enqueue(placeTank(tank));
    }

    // load as many of current physical objects as possible into array
    public int physicals(Physical[] physicals) {
        synchronized (currentPhysicals) {
            try {
                int size = currentPhysicals.size();
                int limit = physicals.length - 1;
                int i;

                for (i = size - 1; i > limit; i--)
                    currentPhysicals.requeue();

                for (; i >= 0; i--)
                    physicals[i] = (Physical) currentPhysicals.requeue();

                return size;
            }
            catch (QueueEmpty e) {
                System.err.println("report to programmer - physicals");

                return 0;
            }
        }
    }

    // search for Tanks within the physicals point of view
    public boolean searchAngle(Physical p, float start, float finish,
                               int distance) {
        int x = p.x();
        int y = p.y();
        int X;
        int Y;
        float a;
        int d = sqr(distance);

        start = normalise(start);
        finish = normalise(finish);

        for (int i = currentPhysicals.size() - 1; i >= 0; i--) {
            if (physicals[i] == p)
                continue;

            if (physicals[i] instanceof Shell)
                continue;

            X = physicals[i].x();
            Y = physicals[i].y();

            if (X != x)
                a = atan((float) (X - x) / (float) (y - Y));
            else
                a = Y < y ? 0 : PI;

            if (y < Y)
                a += PI;

            a = normalise(a);

            if (less(start, a) && less(a, finish)
                    && sqr(X - x) + sqr(Y - y) < d &&!bspt.in(x, y, X, Y))
                return true;
        }

        return false;
    }

    // pmarco@ipvvis.unipv.it
    // Returns a snapshot array of current physicals.
    public Physical[] getCurrentPhysicals() {
        Physical[] physicals = new Physical[currentPhysicals.size()];

        physicals(physicals);

        return physicals;
    }

    // check for line collision with landscape polygons
    public boolean checkLine(int x1, int y1, int x2, int y2) {
        return bspt.in(x1, y1, x2, y2);
    }

    // try and add new phyiscal to world, may cause collision
    public void addPhysical(Physical p) {
        newPhysicals.enqueue(p);

        if (bspt.in(p.x(), p.y(), p.radius()))
            p.collision(true);
    }

    // try and remove physical from world,
    // have to check current and new physicals
    public void subPhysical(Physical p) {
        try {
            oldPhysicals.enqueue(p);
            currentPhysicals.remove(p);

            if (p instanceof Tank) {
                Tank t = (Tank) p;

                t.stop();
                t.setSurvival(tickCounter - 1);
                transcript.append("Tank " + t.getName() + " removed at tick "
                                  + tickCounter + ".\n");
                oldTanks.enqueue(p);
            }
        }
        catch (QueueItemAbsent e) {
            try {
                newPhysicals.remove(p);
            }
            catch (QueueItemAbsent f) {
                if (p instanceof Tank)
                    System.err.println("problem tank");
                else
                    System.err.println("problem shell");
            }
        }
    }

}

