/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.np;

import ec.*;
import ec.util.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

/**
 * An ADFArgument is a NPNode which represents an ADF's
 * <i>argument terminal</i>, its counterpart which returns argument
 * values in its associated function tree.  In lil-gp this is called an
 * ARG node.
 *
 * <p>Obviously, if you have Argument Terminals in a tree, that tree must
 * be only callable by ADFs and ADMs, otherwise the Argument Terminals
 * won't have anything to return.  Furthermore, you must make sure that
 * you don't have an Argument Terminal in a tree whose number is higher
 * than the smallest arity (number of arguments) of a calling ADF or ADM.
 *
 * <p><b>Parameters</b><br>
 * <table>
 * <tr><td valign=top><i>base</i>.<tt>arg</tt><br>
 * <font size=-1>int &gt;= 0</font></td>
 * <td valign=top>(The related argument position for the ADF Argument Node in the associated ADF)</td></tr>
 * </table>
 * <p><b>Default Base</b><br>
 * gp.adf-argument
 */

public class ADFArgument extends NPNode {
    static final long serialVersionUID = -5707087130456738211L;

    public static final String P_ADFARGUMENT = "adf-argument";
    public final static String P_ARGUMENT = "arg";
    int argument;

    public Parameter defaultBase() {
        return NPDefaults.base().push(P_ADFARGUMENT);
    }


    public void setup(final EvolutionState state, final Parameter base) {
        super.setup(state, base);

        Parameter def = defaultBase();

        // make sure we don't have any children...
        if (links.length != 0)
            state.output.error("Incorrect number of links for ADF Argument terminal"
                               + " -- should be 0.  Check the constraints.",
                               base, def);

        argument = state.parameters.getInt(base.push(P_ARGUMENT),
                                           def.push(P_ARGUMENT), 0);

        if (argument < 0)
            state.output.fatal("Argument terminal must have a positive argument number.",
                               base.push(P_ARGUMENT), def.push(P_ARGUMENT));
    }

    public String toString() {
        return "ARG[" + argument + "]";
    }


    public void eval(final EvolutionState state, final int thread,
                     final NPData input, final ADFContext context,
                     final NPMemory newMemory, final NPIndividual individual,
                     final Problem problem) {

        if (context == null)// uh oh
            state.output.fatal("No context with which to evaluate ADFArgument terminal "
                               + toStringForError()
                               + ".  This often happens if you evaluate a tree by hand "
                               + "which is supposed to only be an ADF's associated tree.");

        // Copy memory's contents into newMemory
        context.get(state, thread, input, individual,
                    argument).copyTo(newMemory);
    }

}

