/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.np;

import ec.*;
import ec.util.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

/**
 * ADFContext is the object pushed onto an ADF stack which represents
 * the current context of an ADF function call, that is, how to
 * get the argument values that argument_terminals need to return.
 * *
 * <p>You set up the ADFContext object for a given ADF node with
 * the prepare(...) functions.
 *
 * <p>To evaluate an argument number from an ADFContext, call evaluate(...),
 * and the results are evaluated and copied into input.
 *
 * <p><b>Parameters</b><br>
 * <table>
 * <tr><td valign=top><i>base</i><tt>.data</tt><br>
 * <font size=-1>classname, inherits and != ec.NPData</font></td>
 * <td valign=top>(the class for the ADFContext's basic NPData type --
 * typically this is the same as NPProblem's NPData type)</td></tr>
 * </table>
 * <p><b>Default Base</b><br>
 * gp.adf-context
 * <p><b>Parameter bases</b><br>
 * <table>
 * <tr><td valign=top><i>base</i><tt>.data</tt><br>
 * <td valign=top>(the ADFContext's basic NPData type)</td></tr>
 * </table>
 */

public class ADFContext implements Prototype {
    static final long serialVersionUID = -96886176762694121L;

    public final static String P_ADFCONTEXT = "adf-context";

    public final static int INITIAL_ARGUMENT_SIZE = 2;  // seems reasonable

    /**
     * The ADF node proper
     */
    public ADF adf;

    /**
     * An array of NPData nodes (none of the null, when it's used)
     * holding an ADF's arguments' return results
     */
    public NPMemory[] arguments;

    public Parameter defaultBase() {
        return NPDefaults.base().push(P_ADFCONTEXT);
    }

    public void setup(final EvolutionState state, final Parameter base) {

        // Do nothing
    }

    public Object protoClone() throws CloneNotSupportedException {
        ADFContext myobj = (ADFContext) (super.clone());

        if (arguments != null) {

            // deep-clone the contexts
            myobj.arguments = new NPMemory[arguments.length];

            for (int x = 0; x < myobj.arguments.length; x++) {

                // Warning! Memories are just referenced, not copied
                myobj.arguments[x] = arguments[x];
            }
        }

        return myobj;
    }

    public final Object protoCloneSimple() {
        try {
            return protoClone();
        }
        catch (CloneNotSupportedException e) {
            throw new InternalError();
        }   // never happens
     }


    /**
     * Get the argument number in the current context
     */
    public NPMemory get(final EvolutionState state, final int thread,
                        final NPData input, final NPIndividual individual,
                        final int argument) {

        // do I have that many arguments?
        if (argument >= adf.links.length || argument < 0) {
            individual.printIndividual(state, 0, 3000);
            state.output.fatal("Invalid argument number for "
                               + adf.errorInfo());
        }

        if (adf == null) {
            state.output.fatal("ADF is null for " + adf.errorInfo());
        }

        return arguments[argument];
    }


    /**
     * Increases arguments to accommodate space if necessary.
     * Sets adf to a.
     */
    public final void prepare(ADF adfNode) {

        arguments = new NPMemory[adfNode.links.length];

        for (int i = 0; i < arguments.length; i++)
            arguments[i] = adfNode.links[i].memory;


        adf = adfNode;
    }

}

