/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.np;

import ec.*;
import ec.util.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

/**
 * ERC is an abstract NPNode which implements Ephemeral Random Constants,
 * as described in Koza I.  An ERC is a node which, when first instantiated,
 * gets set to some random constant value which it always returns from
 * then on, even after being crossed over into other individuals.
 * In order to implement an ERC, you need to override several methods below.
 *
 * <p> Remember that if your ERC value isn't a simple or mutable type
 * (like an int or a string), you'll have to deep-clone it in an overridden
 * protoClone() method.
 *
 * @author Sean Luke
 * @version 1.0
 */

public abstract class ERC extends NPNode {
    static final long serialVersionUID = 2775358051008998269L;

    public static final String ERC_PREFIX = "ERC";

    /**
     * Returns the lowercase "name" of this ERC function class, some
     * simple, short name which distinguishes this class from other ERC
     * function classes you're using.  If you have only one ERC function
     * class, you can just return "".  Whatever the name is, it should
     * generally only have letters, numbers, or hyphens or underscores in it.
     * No whitespace or other characters.
     */
    public abstract String name();

    /**
     * Checks to make certain that the ERC has no children.
     */
    public void checkConstraints(final EvolutionState state, final int tree,
                                 final NPIndividual typicalIndividual,
                                 final Parameter individualBase) {
        super.checkConstraints(state, tree, typicalIndividual,
                               individualBase);

        // make sure we don't have any children...
        if (links.length != 0)
            state.output.error("Incorrect number of children for the node "
                               + toStringForError() + " (should be 0)");
    }

    /**
     * Remember to override this to randomize your ERC after it has been cloned.
     * The prototype will not ever receive this method call.
     */
    public abstract void resetNode(final EvolutionState state, int thread);

    /**
     * Implement this to do ERC-to-ERC comparisons.
     */
    public abstract boolean nodeEquals(final NPNode node);

    /**
     * Implement this to hash ERCs, along with other nodes, in such a way that two
     * "equal" ERCs will usually hash to the same value.
     */

    public abstract int nodeHashCode();

    /**
     * You might want to override this to return a special human-readable
     * version of the erc value; otherwise this defaults to toString();
     * This should be something that resembles a LISP atom.
     * If a simple number or other object won't suffice, you might use something
     * that begins with ERC_PREFIX + name() + [ + ... + ]
     */

    public String toStringForHumans() {
        return toString();
    }

    /**
     * This defaults to simply ERC_PREFIX + name()
     */

    public String toString() {
        return ERC_PREFIX + name();
    }

}

