/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.np;

import ec.simple.SimpleInitializer;
import ec.util.Parameter;
import ec.EvolutionState;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

/**
 * NPInitializer is a SimpleInitializer which sets up all the Cliques,
 * ( the initial
 * [tree/node]constraints, types, and function sets) for the NP system.
 *
 * <p>Note that the Cliques must be set up in a very particular order:
 * <ol><li>NPType</li><li>NPNodeConstraints</li><li>NPFunctionSets</li><li>NPTreeConstraints</li></ol>
 * <p><b>Parameter bases</b><br>
 * <table>
 * <tr><td valign=top><tt>np.type</tt></td>
 * <td>NPTypes</td></tr>
 * <tr><td valign=top><tt>np.nc</tt></td>
 * <td>NPNodeConstraints</td></tr>
 * <tr><td valign=top><tt>np.gc</tt></td>
 * <td>NPGraphConstraints</td></tr>
 * <tr><td valign=top><tt>np.fs</tt></td>
 * <td>NPFunctionSets</td></tr>
 * </table>
 * @author Sean Luke
 * @version 1.0
 */

public class NPInitializer extends SimpleInitializer {
    static final long serialVersionUID = 2465343549434995938L;

    public final static String P_TYPE = "type";
    public final static String P_NODECONSTRAINTS = "nc";
    public final static String P_GRAPHCONSTRAINTS = "gc";
    public final static String P_FUNCTIONSETS = "fs";

    public void setup(final EvolutionState state, final Parameter base) {
        super.setup(state, base);

        // This is a good place to set up the types.  We use our own base off the
        // default NP base. This MUST be done before loading constraints.
        NPType.setupTypes(state, NPDefaults.base().push(P_TYPE));

        // Now let's load our constraints and function sets also.
        // This is done in a very specific order, don't change it or things
        // will break.
        NPNodeConstraints.setupConstraints(state,
                                           NPDefaults.base().push(P_NODECONSTRAINTS));
        NPFunctionSet.setupFunctionSets(state,
                                        NPDefaults.base().push(P_FUNCTIONSETS));
        NPGraphConstraints.setupConstraints(state,
                                           NPDefaults.base().push(P_GRAPHCONSTRAINTS));
    }

}

