/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.np;

import java.io.*;

import ec.*;
import ec.util.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */


/**
 * NPSpecies is a simple individual which is suitable as a species
 * for NP subpopulations.  NPSpecies' individuals must be NPIndividuals,
 * and often their pipelines are NPBreedingPipelines (at any rate,
 * the pipelines will have to return members of NPSpecies!).
 *
 * <p><b>Default Base</b><br>
 * np.species
 *
 */

public class NPSpecies extends Species implements NPSpeciesForm {
    static final long serialVersionUID = 70196400570316880L;

    public static final String P_NPSPECIES = "species";

    public Parameter defaultBase() {
        return NPDefaults.base().push(P_NPSPECIES);
    }

    public void setup(final EvolutionState state, final Parameter base) {
        super.setup(state, base);

        // check to make sure that our individual prototype is a NPIndividual
        if (!(i_prototype instanceof NPIndividual))
            state.output.fatal("The Individual class for the Species "
                               + getClass().getName()
                               + " is must be a subclass of ec.gp.NPIndividual.", base);
    }

    public Individual newIndividual(EvolutionState state,
                                    Subpopulation _population,
                                    Fitness _fitness) throws CloneNotSupportedException {
        NPIndividual newind = (NPIndividual) (i_prototype.protoClone());

        // Initialize the graphs
        for (int x = 0; x < newind.graphs.length; x++)
            newind.graphs[x].buildGraph(state, 0); // unthreaded, right?

        // Set the fitness
        newind.fitness = _fitness;
        newind.evaluated = false;

        // ...and we're ready!
        return newind;
    }

    public Individual newIndividual(final EvolutionState state,
                                    final Subpopulation _population,
                                    final Fitness _fitness, final int thread,
                                    final LineNumberReader reader) throws IOException,
                                    CloneNotSupportedException {
        NPIndividual newind = (NPIndividual) (i_prototype.protoClone());

        // Set the fitness -- must be done BEFORE loading!
        newind.fitness = _fitness;
        newind.evaluated = false;

        // load that sucker
        newind.readIndividual(state, thread, reader);

        // and we're ready!
        return newind;
    }

}

