/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.np;

import ec.*;
import ec.gp.koza.KozaFitness;
import ec.util.*;
import ec.simple.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class NPStatistics extends SimpleStatistics {
    static final long serialVersionUID = -1987749131360763532L;

    public static String P_FULL = "gather-full";

    boolean doFull;

    // total number of individuals
    long numInds;

    // timings
    long lastTime;
    long initializationTime;
    long breedingTime;
    long evaluationTime;
    long nodesInitialized;
    long nodesEvaluated;
    long nodesBred;

    // memory usage info
    long lastUsage = 0;
    long initializationUsage = 0;
    long breedingUsage = 0;
    long evaluationUsage = 0;

    public void setup(final EvolutionState state, final Parameter base) {
        super.setup(state, base);

        doFull = state.parameters.getBoolean(base.push(P_FULL), null, false);
        nodesInitialized = nodesEvaluated = nodesBred = 0;
        breedingTime = evaluationTime = 0;
    }


    public void preInitializationStatistics(final EvolutionState state) {
        if (doFull) {
            Runtime r = Runtime.getRuntime();

            lastTime = System.currentTimeMillis();
            lastUsage = r.totalMemory() - r.freeMemory();
        }
    }

    public void postInitializationStatistics(final EvolutionState state) {

        // gather timings
        if (doFull) {
            Runtime r = Runtime.getRuntime();
            long curU = r.totalMemory() - r.freeMemory();

            if (curU > lastUsage)
                initializationUsage = curU - lastUsage;

            initializationTime = System.currentTimeMillis() - lastTime;

            // Determine how many nodes we have
            for (int x = 0; x < state.population.subpops.length; x++) {

                // check to make sure they're the right class
                if (!(state.population.subpops[x].species
                        instanceof NPSpeciesForm))
                    state.output.fatal("Subpopulation " + x
                                       + " is not of the species form NPSpeciesForm."
                                       + "  Cannot do timing statistics with KozaStatistics.");

                for (int y = 0;
                     y < state.population.subpops[x].individuals.length;
                     y++) {
                    NPIndividual i =
                        (NPIndividual) (state.population.subpops[x].individuals[y]);

                    for (int z = 0; z < i.graphs.length; z++)
                        nodesInitialized +=
                            i.graphs[z].nodes.length;
                }
            }
        }
    }

    public void preBreedingStatistics(final EvolutionState state) {
        if (doFull) {
            Runtime r = Runtime.getRuntime();

            lastTime = System.currentTimeMillis();
            lastUsage = r.totalMemory() - r.freeMemory();
        }
    }

    public void postBreedingStatistics(final EvolutionState state) {

        // gather timings
        if (doFull) {
            Runtime r = Runtime.getRuntime();
            long curU = r.totalMemory() - r.freeMemory();

            if (curU > lastUsage)
                breedingUsage += curU - lastUsage;

            breedingTime += System.currentTimeMillis() - lastTime;

            // Determine how many nodes we have
            for (int x = 0; x < state.population.subpops.length; x++) {

                // check to make sure they're the right class
                if (!(state.population.subpops[x].species
                        instanceof NPSpeciesForm))
                    state.output.fatal("Subpopulation " + x
                                       + " is not of the species form NPSpeciesForm."
                                       + "  Cannot do timing statistics with KozaStatistics.");

                for (int y = 0;
                     y < state.population.subpops[x].individuals.length;
                     y++) {
                    NPIndividual i =
                        (NPIndividual) (state.population.subpops[x].individuals[y]);

                    for (int z = 0; z < i.graphs.length; z++)
                        nodesBred +=
                            i.graphs[z].nodes.length;
                }
            }
        }
    }

    public void preEvaluationStatistics(final EvolutionState state) {
        if (doFull) {
            Runtime r = Runtime.getRuntime();

            lastTime = System.currentTimeMillis();
            lastUsage = r.totalMemory() - r.freeMemory();
        }
    }

    public void postEvaluationStatistics(final EvolutionState state) {

        // Gather statistics


        Runtime r = Runtime.getRuntime();
        long curU = r.totalMemory() - r.freeMemory();

        if (curU > lastUsage)
            evaluationUsage += curU - lastUsage;

        if (doFull)
            evaluationTime += System.currentTimeMillis() - lastTime;


        state.output.println("\n\n\nGeneration " + state.generation
                             + "\n================", Output.V_NO_GENERAL,
                             statisticslog);

        Individual best_i = null;   // quiets compiler complaints

        for (int x = 0; x < state.population.subpops.length; x++) {
            state.output.println("\nSubpopulation " + x
                                 + "\n----------------", Output.V_NO_GENERAL,
                                 statisticslog);

            // gather timings
            if (doFull) {
                long totNodesPerGen = 0;

                // check to make sure they're the right class
                if (!(state.population.subpops[x].species
                        instanceof NPSpeciesForm))
                    state.output.fatal("Subpopulation " + x
                                       + " is not of the species form NPSpeciesForm."
                                       + "  Cannot do timing statistics with KozaStatistics.");

                long[] numNodes =
    new long[((NPIndividual) (state.population.subpops[x].species.i_prototype)).graphs.length];

                for (int y = 0;
                        y < state.population.subpops[x].individuals.length;
                        y++) {
                    NPIndividual i =
                        (NPIndividual) (state.population.subpops[x].individuals[y]);

                    for (int z = 0; z < i.graphs.length; z++) {
                        nodesEvaluated +=
                            i.graphs[z].nodes.length;
                        numNodes[z] +=
                            i.graphs[z].nodes.length;
                    }
                }

                for (int tr = 0; tr < numNodes.length; tr++)
                    totNodesPerGen += numNodes[tr];

                state.output.println("Avg Nodes: "
                                     + ((double) totNodesPerGen)
                                     / state.population.subpops[x].individuals.length,
                                     Output.V_NO_GENERAL, statisticslog);
                state.output.print("Nodes/graph: [", Output.V_NO_GENERAL,
                                   statisticslog);

                for (int tr = 0; tr < numNodes.length; tr++) {
                    if (tr > 0)
                        state.output.print("|", Output.V_NO_GENERAL,
                                           statisticslog);

                    state.output.print(""
                                       + ((double) numNodes[tr])
                                       / state.population.subpops[x].individuals.length,
                                       Output.V_NO_GENERAL, statisticslog);
                }

                state.output.println("]", Output.V_NO_GENERAL, statisticslog);
            }


            float meanRaw = 0.0f;
            float meanAdjusted = 0.0f;
            long hits = 0;

            if (!(state.population.subpops[x].f_prototype
                    instanceof KozaFitness))
                state.output.fatal("Subpopulation " + x
                                   + " is not of the fitness KozaFitness.  Cannot do timing statistics with KozaStatistics.");


            best_i = state.population.subpops[x].individuals[0];

            for (int y = 0;
                    y < state.population.subpops[x].individuals.length; y++) {

                // best individual
                if (state.population.subpops[x].individuals[y].fitness.betterThan(best_i.fitness))
                    best_i = state.population.subpops[x].individuals[y];

                    // mean for population

                meanRaw +=
                    ((KozaFitness) (state.population.subpops[x].individuals[y].fitness)).rawFitness();
                meanAdjusted +=
                    ((KozaFitness) (state.population.subpops[x].individuals[y].fitness)).adjustedFitness();
                hits +=
                    ((KozaFitness) (state.population.subpops[x].individuals[y].fitness)).hits;
            }

            // compute fitness stats
            meanRaw /= state.population.subpops[x].individuals.length;
            meanAdjusted /= state.population.subpops[x].individuals.length;

            state.output.print("Mean fitness raw: " + meanRaw + " adjusted: "
                               + meanAdjusted + " hits: "
                               + ((double) hits)
                               / state.population.subpops[x].individuals.length,
                               Output.V_NO_GENERAL, statisticslog);

            state.output.println("", Output.V_NO_GENERAL, statisticslog);

            // compute inds stats
            numInds += state.population.subpops[x].individuals.length;
        }

        // now test to see if it's the new best_of_run
        if (best_of_run == null
                || best_i.fitness.betterThan(best_of_run.fitness))
            best_of_run = best_i;

            // print the best-of-generation individual

        state.output.println("Best Individual of Generation:",
                             Output.V_NO_GENERAL, statisticslog);
        best_i.printIndividualForHumans(state, statisticslog,
                                        Output.V_NO_GENERAL);
    }






    /**
     * Logs the best individual of the run.
     */
    public void finalStatistics(final EvolutionState state,
                                final int result) {
        state.output.println("\n\n\nFinal Statistics\n================",
                             Output.V_NO_GENERAL, statisticslog);

        state.output.println("Total Individuals Evaluated: " + numInds,
                             Output.V_NO_GENERAL, statisticslog);

        // for now we just print the best fitness

        state.output.println("\nBest Individual of Run:",
                             Output.V_NO_GENERAL, statisticslog);
        best_of_run.printIndividualForHumans(state, statisticslog,
                                             Output.V_NO_GENERAL);


        // Output timings
        if (doFull) {
            state.output.println("\n\n\nTimings\n=======",
                                 Output.V_NO_GENERAL, statisticslog);

            state.output.println("Initialization: "
                                 + ((float) initializationTime) / 1000
                                 + " secs total, " + nodesInitialized
                                 + " nodes, "
                                 + nodesInitialized
                                 / (((float) initializationTime) / 1000)
                                 + " nodes/sec", Output.V_NO_GENERAL,
                                                 statisticslog);
            state.output.println("Evaluating: "
                                 + ((float) evaluationTime) / 1000
                                 + " secs total, " + nodesEvaluated
                                 + " nodes, "
                                 + nodesEvaluated
                                 / (((float) evaluationTime) / 1000)
                                 + " nodes/sec", Output.V_NO_GENERAL,
                                                 statisticslog);
            state.output.println("Breeding: " + ((float) breedingTime) / 1000
                                 + " secs total, " + nodesBred + " nodes, "
                                 + nodesBred
                                 / (((float) breedingTime) / 1000)
                                 + " nodes/sec", Output.V_NO_GENERAL,
                                                 statisticslog);

            state.output.println("\n\n\nMemory Usage\n==============",
                                 Output.V_NO_GENERAL, statisticslog);
            state.output.println("Initialization: "
                                 + ((float) initializationUsage) / 1024
                                 + " KB total, " + nodesInitialized
                                 + " nodes, "
                                 + nodesInitialized
                                 / (((float) initializationUsage) / 1024)
                                 + " nodes/KB", Output.V_NO_GENERAL,
                                                statisticslog);
            state.output.println("Evaluating: "
                                 + ((float) evaluationUsage) / 1024
                                 + " KB total, " + nodesEvaluated
                                 + " nodes, "
                                 + nodesEvaluated
                                 / (((float) evaluationUsage) / 1024)
                                 + " nodes/KB", Output.V_NO_GENERAL,
                                                statisticslog);
            state.output.println("Breeding: "
                                 + ((float) breedingUsage) / 1024
                                 + " KB total, " + nodesBred + " nodes, "
                                 + nodesBred
                                 / (((float) breedingUsage) / 1024)
                                 + " nodes/KB", Output.V_NO_GENERAL,
                                                statisticslog);
        }

    }

}

