/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.np;

import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;

import ec.*;
import ec.util.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */


/**
 * NPType is a Clique which represents types in
 * Just a placeholder for now. We do not know yet whether strongly-typed
 * NP programming will be implemented at all
 */

public class NPType implements Clique {
    static final long serialVersionUID = -7920752003722356084L;

    public final static String P_NAME = "name";
    public final static String P_SIZE = "size";

    /**
     * The name of the type
     */
    public String name;

    /**
     * The preassigned integer value for the type
     */
    public int type;

    /** The number of built-in types */
    public static int numTypes;

    /**
     * Am I compatible with ("fit" with) <i>t</i>?  For two atomic
     * types, this is done by direct pointer equality.  For
     * two set types, this is done by determining if the intersection
     * is nonempty.  A set type is compatible with an atomic type
     * if it contains the atomic type in its set.
     */
    public boolean compatibleWith(final NPType t) {
        return true;
    }

    /**
     * Returns the type's name
     */
    public String toString() {
        return name;
    }

    /**
     * A global storage facility for all known NPTypes.
     */
    public static Hashtable all;

    static {
        all = new Hashtable();
        numTypes = 0;
    }

    /**
     * Sets up all the types, loading them from the parameter file.  This
     * must be called before anything is called which refers to a type by
     * name.
     */

    public static void setupTypes(final EvolutionState state,
                                  final Parameter base) {
        state.output.message("Processing NP Types");

        // How many built-in types do we have?
        int x = state.parameters.getInt(base.push(P_SIZE), null, 1);

        if (x <= 0)
            state.output.fatal("The number of NP built-in types must be at least 1.",
                               base.push(P_SIZE));


        for (int y = 0; y < x; y++)
            new NPType().setup(state, base.push("" + y));


        // Postprocess the types
        postProcessTypes();
    }

    public void setup(final EvolutionState state, final Parameter base) {

        // What's my name?
        name = state.parameters.getString(base.push(P_NAME), null);

        if (name == null)
            state.output.fatal("No name was given for this NP type.",
                               base.push(P_NAME));

            // Register me

        NPType old_type = (NPType) (all.put(name, this));

        if (old_type != null)
            state.output.fatal("The NP type \"" + name
                               + "\" has been defined multiple times.", base.push(P_NAME));
    }

    /**
     * Returns a type for a given name.
     * You must guarantee that after calling typeFor(...) one or
     * several times, you call state.output.exitIfErrors() once.
     */

    public static NPType typeFor(final String name,
                                 final EvolutionState state) {
        NPType type = (NPType) (all.get(name));

        if (type == null)
            state.output.error("The NP type \"" + name
                               + "\" could not be found.");

        return type;
    }

    public static void postProcessTypes() {

        // assign positive integers and 0 to built-in types
        int x = 0;
        Enumeration e = all.elements();

        while (e.hasMoreElements()) {
            NPType t = (NPType) (e.nextElement());

                t.type = x;
                x++;
        }

        // at this point, x holds the number of built-in types.
        numTypes = x;
    }

    public final void writeObject(ObjectOutputStream out) throws IOException {

        // this wastes some data, but what the heck. It's not a whole lot of types.

        out.defaultWriteObject();
        out.writeObject(all);
        out.writeInt(numTypes);
    }

    public final void readObject(ObjectInputStream in)
            throws IOException, ClassNotFoundException {
        in.defaultReadObject();

        all = (Hashtable) (in.readObject());
        numTypes = in.readInt();
    }

}

