/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.np.breed;

import java.util.Vector;
import java.io.Serializable;

import ec.*;
import ec.util.*;

import evision.ec.np.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class NPCrossoverPipeline extends BreedingPipeline {
    static final long serialVersionUID = 4736501465592084720L;

    public static final String P_CROSSOVER = "xover";
    public static final String P_UNCONNECTED = "unconnectedNodes";
    public static final String P_TOSS = "toss";
    public static final int INDS_PRODUCED = 2;
    public static final int NUM_SOURCES = 2;

    /**
     * Whether second offspring is to be tossed
     */
    public boolean tossSecondParent;

    /**
     * Whether unconnected nodes have to be collected
     */
    public boolean unconnectedNodes;

    NPIndividual[] parents = new NPIndividual[2];
    NPIndividual[] newInds = new NPIndividual[2];

    public Parameter defaultBase() {
        return NPBreedDefaults.base().push(P_CROSSOVER);
    }

    public int numSources() {
        return NUM_SOURCES;
    }

    public void setup(final EvolutionState state, final Parameter base) {
        super.setup(state, base);

        Parameter def = defaultBase();

        tossSecondParent = state.parameters.getBoolean(base.push(P_TOSS),
                def.push(P_TOSS), false);

        unconnectedNodes =
            state.parameters.getBoolean(base.push(P_UNCONNECTED),
                                        def.push(P_UNCONNECTED), true);
    }

    /**
     * Returns 2 (unless tossing the second sibling, in which case it returns 1)
     */
    public int typicalIndsProduced() {
        return (tossSecondParent ? 1 : INDS_PRODUCED);
    }


    public int produce(final int min, final int max, final int start,
                       final int subpopulation, final Individual[] inds,
                       final EvolutionState state,
                       final int thread) throws CloneNotSupportedException {

        MersenneTwisterFast random = state.random[thread];

        // how many individuals should we make?
        int n = tossSecondParent ? 1 : INDS_PRODUCED;

        if (n < min)
            n = min;

        if (n > max)
            n = max;

        for (int q = start; q < start + n; ) {

            // keep on going until we're filled up

            // grab two individuals from our sources

            if (sources[0] == sources[1]) {

                // grab from the same source
                sources[0].produce(2, 2, 0, subpopulation, parents, state,
                                   thread);
            }
            else {

                // grab from different sources
                sources[0].produce(1, 1, 0, subpopulation, parents, state,
                                   thread);
                sources[1].produce(1, 1, 1, subpopulation, parents, state,
                                   thread);
            }

            if ((sources[0] instanceof BreedingPipeline)) {

                // They are copies, already
                for (int i = 0; i < newInds.length; i++) {
                    newInds[i] = parents[i];
                }
            }
            else {

                // Ok, deep clone them
                for (int i = 0; i < newInds.length; i++) {
                    newInds[i] = (NPIndividual) parents[i].protoClone();
                }
            }

            // Assume that they come from the same species, i.e. all inds have
            // the same number of graphs

            int selectedGraph = random.nextInt(newInds[0].graphs.length);

            // Make the shorter of two graphs to be in newInds[0]
            if (newInds[0].graphs[selectedGraph].nodes.length
                    > newInds[1].graphs[selectedGraph].nodes.length) {

                // Swap
                NPIndividual swap = newInds[0];

                newInds[0] = newInds[1];
                newInds[1] = swap;
            }

            NPGraph[] graphs = new NPGraph[2];

            graphs[0] = newInds[0].graphs[selectedGraph];
            graphs[1] = newInds[1].graphs[selectedGraph];

            // Select gap
            int gap = 0;

            if (graphs[0].nodes.length != graphs[1].nodes.length) {
                gap = random.nextInt(graphs[1].nodes.length
                                     - graphs[0].nodes.length + 1);
            }

            // Select crossover point
            int xover = gap + random.nextInt(graphs[0].nodes.length);

            // Create first offspring
            Vector temp = new Vector();

            for (int i = 0; i < xover - gap; i++) {
                temp.addElement(graphs[0].nodes[i]);
            }

            for (int i = xover; i < graphs[1].nodes.length; i++) {
                temp.addElement(graphs[1].nodes[i]);
            }

            NPNode[][] nodes = new NPNode[2][];

            nodes[0] = new NPNode[temp.size()];

            temp.toArray(nodes[0]);

            // Fix links
            for (int i = 0; i < xover - gap; i++) {
                for (int j = 0; j < nodes[0][i].links.length; j++) {
                    if (nodes[0][i].links[j].position < xover - gap) {

                        // It's ok
                    }
                    else {

                        // Keep position, it's safe
                        nodes[0][i].links[j] =
                            nodes[0][nodes[0][i].links[j].position];
                    }
                }
            }

            for (int i = xover - gap; i < nodes[0].length; i++) {
                for (int j = 0; j < nodes[0][i].links.length; j++) {
                    if (nodes[0][i].links[j].position < xover) {
                        if (nodes[0][i].links[j].position >= gap) {

                            // Keep position after a left shift by gap
                            nodes[0][i].links[j] =
                                nodes[0][nodes[0][i].links[j].position - gap];
                        }
                        else {

                            // Just re-assign
                            nodes[0][i].links[j] =
                                nodes[0][random.nextInt(nodes[0].length)];
                        }
                    }
                    else {

                        // It's ok
                    }
                }
            }

            // Create second offspring
            temp.clear();

            for (int i = 0; i < xover; i++) {
                temp.addElement(graphs[1].nodes[i]);
            }

            for (int i = xover - gap; i < graphs[0].nodes.length; i++) {
                temp.addElement(graphs[0].nodes[i]);
            }

            nodes[1] = new NPNode[temp.size()];

            temp.toArray(nodes[1]);

            // Fix links
            for (int i = 0; i < xover; i++) {
                for (int j = 0; j < nodes[1][i].links.length; j++) {
                    if (nodes[1][i].links[j].position < xover) {

                        // It's ok
                    }
                    else {
                        if (nodes[1][i].links[j].position < nodes[1].length) {

                            // Keep position
                            nodes[1][i].links[j] =
                                nodes[1][nodes[1][i].links[j].position];
                        }
                        else {

                            // Just re-assign
                            nodes[1][i].links[j] =
                                nodes[1][random.nextInt(nodes[1].length)];
                        }
                    }
                }
            }

            for (int i = xover; i < nodes[1].length; i++) {
                for (int j = 0; j < nodes[1][i].links.length; j++) {
                    if (nodes[1][i].links[j].position < xover - gap) {

                        // Keep position after a right shift by gap
                        nodes[1][i].links[j] =
                            nodes[1][nodes[1][i].links[j].position + gap];
                    }
                    else {

                        // It's ok
                    }
                }
            }

            // Fix owners and positions
            for (int i = 0; i < nodes[0].length; i++) {
                nodes[0][i].position = i;
                nodes[0][i].owner = graphs[0];
            }

            for (int i = 0; i < nodes[1].length; i++) {
                nodes[1][i].position = i;
                nodes[1][i].owner = graphs[1];
            }

            // Update graphs
            newInds[0].graphs[selectedGraph].nodes = nodes[0];
            newInds[1].graphs[selectedGraph].nodes = nodes[1];

            // Collect, if requested
            if (!unconnectedNodes) {
                newInds[0].graphs[selectedGraph].collectUnconnectedNodes();
                newInds[1].graphs[selectedGraph].collectUnconnectedNodes();
            }

            // Make they will be evaluated
            newInds[0].evaluated = false;
            newInds[1].evaluated = false;

            // Debugging
            newInds[0].verifyIndividual(unconnectedNodes);
            newInds[1].verifyIndividual(unconnectedNodes);

            // Copy individuals
            // Make sure that offspring is meaningful (i.e. it does contain
            // at least one node. Otherwise, use parents instead).
            if (nodes[0].length > 0) {
                inds[q] = newInds[0];
            }
            else {

                // Use Parent
                if ((sources[0] instanceof BreedingPipeline)) {
                    inds[q] = parents[0];
                }
                else {
                    inds[q] = (NPIndividual) parents[0].protoClone();
                }
            }

            q++;

            if (q < n + start &&!tossSecondParent) {
                if (nodes[1].length > 0) {
                    inds[q] = newInds[1];
                }
                else {

                    // Use Parent
                    if ((sources[0] instanceof BreedingPipeline)) {
                        inds[q] = parents[1];
                    }
                    else {
                        inds[q] = (NPIndividual) parents[1].protoClone();
                    }
                }

                q++;
            }
        }

        return n;
    }

}

