/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.np.breed;

import java.util.Vector;
import java.io.Serializable;

import ec.*;
import ec.util.*;

import evision.ec.np.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class NPGraftingPipeline extends BreedingPipeline {
    static final long serialVersionUID = 3207969616263924024L;

    public static final String P_GRAFTING = "grafting";
    public static final String P_UNCONNECTED = "unconnectedNodes";
    public static final int INDS_PRODUCED = 1;
    public static final int NUM_SOURCES = 2;

    /**
     * Whether unconnected nodes have to be collected
     */
    public boolean unconnectedNodes;

    NPIndividual[] parents = new NPIndividual[2];
    NPIndividual newInd = null;

    public Parameter defaultBase() {
        return NPBreedDefaults.base().push(P_GRAFTING);
    }

    public int numSources() {
        return NUM_SOURCES;
    }

    public void setup(final EvolutionState state, final Parameter base) {
        super.setup(state, base);

        Parameter def = defaultBase();

        unconnectedNodes =
            state.parameters.getBoolean(base.push(P_UNCONNECTED),
                                        def.push(P_UNCONNECTED), true);
    }

    /**
     * Returns 2 (unless tossing the second sibling, in which case it returns 1)
     */
    public int typicalIndsProduced() {
        return INDS_PRODUCED;
    }


    public int produce(final int min, final int max, final int start,
                       final int subpopulation, final Individual[] inds,
                       final EvolutionState state,
                       final int thread) throws CloneNotSupportedException {

        MersenneTwisterFast random = state.random[thread];

        // how many individuals should we make?
        int n = INDS_PRODUCED;

        if (n < min)
            n = min;

        if (n > max)
            n = max;

        for (int q = start; q < start + n; ) {

            // keep on going until we're filled up

            // grab two individuals from our sources

            if (sources[0] == sources[1]) {

                // grab from the same source
                sources[0].produce(2, 2, 0, subpopulation, parents, state,
                                   thread);
            }
            else {

                // grab from different sources
                sources[0].produce(1, 1, 0, subpopulation, parents, state,
                                   thread);
                sources[1].produce(1, 1, 1, subpopulation, parents, state,
                                   thread);
            }

            if ((sources[0] instanceof BreedingPipeline)) {

                // They are copies, already
                newInd = parents[0];
            }
            else {

                // Ok, deep clone it
                newInd = (NPIndividual) parents[0].protoClone();
            }

            // Assume that they come from the same species, i.e. all inds have
            // the same number of graphs

            int selectedGraph = random.nextInt(newInd.graphs.length);

            NPGraph graph = newInd.graphs[selectedGraph];
            NPGraph otherGraph = parents[1].graphs[selectedGraph];

            if (!(sources[0] instanceof BreedingPipeline)) {

                // Ok, deep clone the graph only
                otherGraph = (NPGraph) otherGraph.protoClone();
            }

            // Create new node array
            NPNode[] nodes =
                new NPNode[graph.nodes.length + otherGraph.nodes.length];

            boolean[] hasNonTerminals = new boolean[2];

            // Copy nodes from first parent
            for (int i = 0; i < graph.nodes.length; i++) {

                nodes[i] = graph.nodes[i];

                if (nodes[i].links.length > 0) {
                    hasNonTerminals[0] = true;
                }
            }

            // Copy nodes from second parent
            for (int i = 0; i < otherGraph.nodes.length; i++) {

                int idx = graph.nodes.length + i;

                nodes[idx] = otherGraph.nodes[i];

                if (nodes[idx].links.length > 0) {
                    hasNonTerminals[1] = true;
                }
            }

            int grafting;
            int graftingLink;

            // Graft first parent, if possible
            if (hasNonTerminals[0]) {

                // Since the graph has non-terminals, this will eventually end
                while (true) {
                    grafting = random.nextInt(graph.nodes.length);

                    if (nodes[grafting].links.length > 0) {
                        break;
                    }
                }

                graftingLink = random.nextInt(nodes[grafting].links.length);
                nodes[grafting].links[graftingLink] =
                    nodes[graph.nodes.length + random.nextInt(otherGraph.nodes.length)];
            }

            // Graft second parent, if possible
            if (hasNonTerminals[1]) {

                // Since the graph has non-terminals, this will eventually end
                while (true) {
                    grafting = graph.nodes.length
                               + random.nextInt(otherGraph.nodes.length);

                    if (nodes[grafting].links.length > 0) {
                        break;
                    }
                }

                graftingLink = random.nextInt(nodes[grafting].links.length);
                nodes[grafting].links[graftingLink] =
                    nodes[random.nextInt(graph.nodes.length)];
            }

            // Fix owners and positions
            for (int i = 0; i < nodes.length; i++) {
                nodes[i].position = i;
                nodes[i].owner = graph;
            }

            // Update graph
            graph.nodes = nodes;

            // Collect, if requested
            if (!unconnectedNodes) {
                graph.collectUnconnectedNodes();
            }

            // Make it will be evaluated
            newInd.evaluated = false;

            // Debugging
            newInd.verifyIndividual(unconnectedNodes);

            // Copy individual
            // Make sure that offspring is meaningful (i.e. it does contain
            // at least one node. Otherwise, use parents instead).
            if (nodes.length > 0) {
                inds[q] = newInd;
            }
            else {

                // Use Parent
                if ((sources[0] instanceof BreedingPipeline)) {
                    inds[q] = parents[0];
                }
                else {
                    inds[q] = (NPIndividual) parents[0].protoClone();
                }
            }

            q++;

        }

        return n;
    }

}

