/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.np.breed;

import java.io.Serializable;

import ec.*;
import ec.util.*;

import evision.ec.np.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class NPMutationPipeline extends BreedingPipeline {
    static final long serialVersionUID = 1261077084979225767L;

    public static final int CHECK_BOUNDARY = 8;

    public static final String P_BUILDER = "build";

    public static final String P_MUTATION = "mutate";
    public static final String P_UNCONNECTED = "unconnectedNodes";
    public static final String P_SWAPARCS = "swaparcs";
    public static final String P_CHANGENODE = "changenode";
    public static final String P_ADDNODE = "addnode";
    public static final String P_DELETENODE = "deletenode";
    public static final String P_MAXSTEPS = "maxsteps";
    public static final String P_PROB = "prob";

    public static final int INDS_PRODUCED = 1;
    public static final int NUM_SOURCES = 1;

    /**
     * How the pipeline builds a new subtree
     */
    public NPGraphBuilder builder;

    /**
     * Whether unconnected nodes have to be collected
     */
    public boolean unconnectedNodes;

    public int maxtries = 3;
    public int maxsteps = 1;

    public float[] opDistribution = {
        0.25f, 0.25f, 0.25f, 0.25f
    };

    public Parameter defaultBase() {
        return NPBreedDefaults.base().push(P_MUTATION);
    }

    public int numSources() {
        return NUM_SOURCES;
    }

    public Object protoClone() throws CloneNotSupportedException {
        NPMutationPipeline c = (NPMutationPipeline) (super.protoClone());

        return c;
    }

    public void setup(final EvolutionState state, final Parameter base) {
        super.setup(state, base);

        Parameter def = defaultBase();

        Parameter p = base.push(P_BUILDER).push("" + 0);
        Parameter d = def.push(P_BUILDER).push("" + 0);

        unconnectedNodes =
            state.parameters.getBoolean(base.push(P_UNCONNECTED),
                                        def.push(P_UNCONNECTED), true);

        builder =
            (NPGraphBuilder) (state.parameters.getInstanceForParameter(p, d,
                NPGraphBuilder.class));

        builder.setup(state, p);

        p = base.push(P_MAXSTEPS);
        maxsteps = state.parameters.getIntWithDefault(p, null, maxsteps);

        RandomChoice.organizeDistribution(opDistribution);
    }

    public int produce(final int min, final int max, final int start,
                       final int subpopulation, final Individual[] inds,
                       final EvolutionState state,
                       final int thread) throws CloneNotSupportedException {

        // how many individuals should we make?
        int n = INDS_PRODUCED;

        if (n < min)
            n = min;

        if (n > max)
            n = max;

            // grab n individuals from our source and stick 'em right into inds.
            // we'll modify them from there

        sources[0].produce(n, n, start, subpopulation, inds, state, thread);

        // now let's mutate 'em
        for (int q = start; q < n + start; q++) {

            // Read individual
            NPIndividual ind = (NPIndividual) inds[q];

            if (!(sources[0] instanceof BreedingPipeline)) {

                // Ok, deep clone it
                ind = (NPIndividual) ind.protoClone();
            }

            // Pick up one graph at random
            NPGraph graph =
                ind.graphs[state.random[thread].nextInt(ind.graphs.length)];

            // How many mutation steps?
            int steps = state.random[thread].nextInt(maxsteps + 1);

            for (; steps > 0; steps--) {

                // Decide mutation method
                int m =
                    RandomChoice.pickFromDistribution(opDistribution,
                                                      state.random[thread].nextFloat(),
                                                      CHECK_BOUNDARY);

                // Mutate it
                switch (m) {

                    case 0:
                        swapArcs(graph, state, thread);

                        break;

                    case 1:
                        changeNode(graph, state, thread);

                        break;

                    case 2:
                        addNode(graph, state, thread);

                        break;

                    case 3:
                        deleteNode(graph, state, thread);

                }

            }

            // Collect, if requested
            if (!unconnectedNodes) {
                graph.collectUnconnectedNodes();
            }

            // Make sure it will be evaluated
            ind.evaluated = false;

            // Debugging
            ind.verifyIndividual(unconnectedNodes);

            // Add the new individual, replacing its previous source
            // Make sure that offspring is meaningful (i.e. it does contain
            // at least one node. Otherwise, use parent instead).
            if (graph.nodes.length > 0) {
                inds[q] = ind;
            }
            else {

                // Use Parent
                if (!(sources[0] instanceof BreedingPipeline)) {

                    // Ok, deep clone it
                    inds[q] = (NPIndividual) inds[q].protoClone();
                }
                else {

                    // It's ok
                }
            }

        }

        return n;
    }

    private void swapArcs(NPGraph graph, EvolutionState state, int thread) {
        int firstNode = 0;
        int firstLink = 0;
        int secondNode = 0;
        int secondLink = 0;
        int i;

        MersenneTwisterFast random = state.random[thread];

        // Select first arc
        for (i = 0; i < maxtries; i++) {
            firstNode = random.nextInt(graph.nodes.length);

            if (graph.nodes[firstNode].links.length > 0)
                break;
        }

        if (i == maxtries)
            return;

        firstLink = random.nextInt(graph.nodes[firstNode].links.length);

        // Select second arc
        for (i = 0; i < maxtries; i++) {
            secondNode = random.nextInt(graph.nodes.length);

            if (graph.nodes[secondNode].links.length > 0)
                break;
        }

        if (i == maxtries)
            return;

        secondLink = random.nextInt(graph.nodes[secondNode].links.length);

        // Ok, swap
        NPNode swap = graph.nodes[firstNode].links[firstLink];
        graph.nodes[firstNode].links[firstLink] =
            graph.nodes[secondNode].links[secondLink];
        graph.nodes[secondNode].links[secondLink] = swap;

    }

    private void changeNode(NPGraph graph, EvolutionState state, int thread) {
        int selectedNode = 0;

        MersenneTwisterFast random = state.random[thread];

        // Select node
        selectedNode = random.nextInt(graph.nodes.length);

        // Create a new node
        NPNode newNode = null;

        try {
            newNode = builder.newGraph(state, graph.constraints().graphtype,
                                       thread, graph,
                                       graph.constraints().functionset,
                                       selectedNode, 1)[0];
        }
        catch (CloneNotSupportedException e) {
            throw new InternalError();
        }   // never happens

        newNode.position = selectedNode;
        newNode.owner = graph;

        // Put node in place and fix its links
        graph.nodes[selectedNode] = newNode;

        for (int i = 0; i < newNode.links.length; i++) {
            newNode.links[i] =
                graph.nodes[random.nextInt(graph.nodes.length)];
        }

        // Fix links from other nodes
        for (int i = 0; i < graph.nodes.length; i++) {
            for (int j = 0; j < graph.nodes[i].links.length; j++) {
                if (graph.nodes[i].links[j].position == selectedNode) {
                    graph.nodes[i].links[j] = newNode;
                }
            }
        }
    }

    private void addNode(NPGraph graph, EvolutionState state, int thread) {
        int addedNode = graph.nodes.length;

        // Make a new, enlarged array and fill it
        NPNode[] newNodes = new NPNode[graph.nodes.length + 1];

        for (int i = 0; i < graph.nodes.length; i++) {
            newNodes[i] = graph.nodes[i];
        }

        // Create new node and fix its links
        try {
            newNodes[addedNode] =
                builder.newGraph(state, graph.constraints().graphtype,
                                 thread, graph,
                                 graph.constraints().functionset, addedNode,
                                 1)[0];
        }
        catch (CloneNotSupportedException e) {
            throw new InternalError();
        }   // never happens


        newNodes[addedNode].position = addedNode;
        newNodes[addedNode].owner = graph;

        for (int i = 0; i < newNodes[addedNode].links.length; i++) {
            newNodes[addedNode].links[i] =
                newNodes[state.random[thread].nextInt(newNodes.length)];
        }

        graph.nodes = newNodes;
    }

    private void deleteNode(NPGraph graph, EvolutionState state, int thread) {
        int selectedNode = 0;

        // Select node
        selectedNode = state.random[thread].nextInt(graph.nodes.length);

        // Make a new, reduced array and fill it
        int j = 0;
        NPNode[] newNodes = new NPNode[graph.nodes.length - 1];

        for (int i = 0; i < graph.nodes.length; i++) {
            if (i != selectedNode) {
                newNodes[j++] = graph.nodes[i];
            }
        }

        // Fix older links
        for (int i = 0; i < newNodes.length; i++) {
            for (j = 0; j < newNodes[i].links.length; j++) {
                if (newNodes[i].links[j] == graph.nodes[selectedNode]) {
                    newNodes[i].links[j] =
                        newNodes[state.random[thread].nextInt(newNodes.length)];
                }
            }

            newNodes[i].position = i;
        }

        graph.nodes = newNodes;
    }

}

