/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.np.build;

import java.util.Hashtable;
import java.util.Enumeration;

import ec.*;
import ec.util.*;

import evision.ec.np.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class NPTypelessBuilder extends NPGraphBuilder {
    static final long serialVersionUID = -5326352921547792974L;

    public static final String P_TYPLESSBUILDER = "typeless";
    public static final String P_UNCONNECTED = "unconnectedNodes";
    public static final String P_MAXDEPTH = "max-size";
    public static final String P_MINDEPTH = "min-size";

    /**
     * The largest maximum graph depth can specify.
     */
    public int maxSize;

    /**
     * The smallest maximum graph depth can specify.
     */
    public int minSize;

    /**
     * Whether unconnected nodes have to be collected
     */
    public boolean unconnectedNodes;

    public Parameter defaultBase() {
        return NPBuildDefaults.base().push(P_TYPLESSBUILDER);
    }

    public void setup(final EvolutionState state, final Parameter base) {
        super.setup(state, base);

        Parameter def = defaultBase();

        unconnectedNodes =
            state.parameters.getBoolean(base.push(P_UNCONNECTED),
                                        def.push(P_UNCONNECTED), true);

        // load maxdepth and mindepth, check that maxdepth>0, mindepth>0, maxdepth>=mindepth
        maxSize = state.parameters.getInt(base.push(P_MAXDEPTH),
                                          def.push(P_MAXDEPTH), 1);

        if (maxSize <= 0)
            state.output.fatal("The Max Size for NPTypelessBuilder must be at least 1.",
                               base.push(P_MAXDEPTH), def.push(P_MAXDEPTH));

        minSize = state.parameters.getInt(base.push(P_MINDEPTH),
                                          def.push(P_MINDEPTH), 1);

        if (minSize <= 0)
            state.output.fatal("The Max Size for NPTypelessBuilder must be at least 1.",
                               base.push(P_MINDEPTH), def.push(P_MINDEPTH));

        if (maxSize < minSize)
            state.output.fatal("Max Size must be >= Min Size for NPTypelessBuilder",
                               base.push(P_MAXDEPTH), def.push(P_MAXDEPTH));

    }



    public NPNode[] newGraph(final EvolutionState state, final NPType type,
                             final int thread, final NPGraph owner,
                             final NPFunctionSet set, final int position,
                             int requestedSize) throws CloneNotSupportedException {


        if (requestedSize == -1) {

            // Assign a random value
            requestedSize =
                state.random[thread].nextInt(maxSize - minSize + 1) + minSize;
        }

        // Create new nodes
        NPNode[] nodes = new NPNode[requestedSize];

        // Create new nodes
        NPFuncInfo[] nn = set.nodes[type.type];

        for (int i = 0; i < requestedSize; i++) {
            nodes[i] =
                (NPNode) nn[state.random[thread].nextInt(nn.length)].node.protoClone();

            // give ERCs a chance to randomize
            nodes[i].resetNode(state, thread);

            nodes[i].position = i;
            nodes[i].owner = owner;
        }

        // Create links
        for (int i = 0; i < requestedSize; i++) {
            for (int j = 0; j < nodes[i].links.length; j++) {
                nodes[i].links[j] =
                    nodes[state.random[thread].nextInt(nodes.length)];
            }
        }

        if (!unconnectedNodes && nodes.length > 1) {

            // Mark unconnected nodes
            boolean[] marks = new boolean[nodes.length];

            for (int i = 0; i < marks.length; i++) {
                marks[i] = false;
            }

            for (int i = 0; i < nodes.length; i++) {

                for (int j = 0; j < nodes[i].links.length; j++) {
                    if (nodes[i].links[j] != nodes[i]) {
                        marks[nodes[i].links[j].position] = true;
                        marks[i] = true;
                    }
                }
            }

            // Replace unconnected nodes with non-terminals
            NPFuncInfo[] nt = set.nonterminalsOverArity[type.type][0];

            for (int i = 0; i < nodes.length; i++) {
                if (marks[i] == false) {
                    nodes[i] =
                        (NPNode) nt[state.random[thread].nextInt(nt.length)].node.protoClone();

                    // give ERCs a chance to randomize
                    nodes[i].resetNode(state, thread);

                    nodes[i].position = i;
                    nodes[i].owner = owner;
                }
            }

            // Set new links
            for (int i = 0; i < nodes.length; i++) {
                if (marks[i] == false) {
                    boolean self = false;

                    // Force the newly-added node to be connected.
                    // Given that this graph is > 1,
                    // this loop must eventually end
                    while (!self) {
                        for (int j = 0; j < nodes[i].links.length; j++) {
                            nodes[i].links[j] =
                                nodes[state.random[thread].nextInt(nodes.length)];

                            if (nodes[i].links[j] != nodes[i]) {
                                self = true;
                            }
                        }
                    }
                }
            }
        }

        return nodes;
    }

}


