/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package evision.ec.util;

import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.OptionalDataException;
import java.io.PrintWriter;

import ec.*;
import ec.util.*;
import ec.simple.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class PseudoEvolve extends Evolve {
    public static final String STAT_FILE = "pseudo-out.stat";


    /**
     * The entry method for an evolutionary run.  Loads either from a checkpoint file,
     * or loads a parameter file and sets up from that.
     */
    public static EvolutionState createState(String[] args) {

        // First we determine if we're loading from a checkpoint file

        EvolutionState state = null;
        ParameterDatabase parameters = null;
        Output output;
        MersenneTwisterFast[] random;
        int[] seeds;
        int breedthreads = 1;
        int evalthreads = 1;
        int verbosity;
        boolean store;
        int x;


        // first we load from a checkpoint file if the user requested
        // that and then we start up from there.

        for (x = 0; x < args.length - 1; x++)
            if (args[x].equals(A_CHECKPOINT)) {
                try {
                    state = Checkpoint.restoreFromCheckpoint(args[x + 1]);

                    state.run(EvolutionState.C_STARTED_FROM_CHECKPOINT);
                }
                catch (OptionalDataException e) {
                    Output.initialError("A ClassNotFoundException was generated upon"
                                        + "starting up from a checkpoint."
                                        + "\nHere it is:\n" + e);
                }
                catch (ClassNotFoundException e) {
                    Output.initialError("A ClassNotFoundException was generated upon"
                                        + "starting up from a checkpoint."
                                        + "\nHere it is:\n" + e);
                }
                catch (IOException e) {
                    Output.initialError("An IO Exception was generated upon"
                                        + "starting up, probably in setting up a log"
                                        + "\nHere it is:\n" + e);
                }

                System.exit(0);
            }

        // at this point, we don't have a checkpoint file so we try
        // reading instead from a parameter file and starting up fresh

        if (state == null)                                  // couldn't find a checkpoint argument
         {

            // 0. find the parameter database
            for (x = 0; x < args.length - 1; x++)
                if (args[x].equals(A_FILE)) {
                    try {
                        parameters = new ParameterDatabase(

                        // not available in jdk1.1: new File(args[x+1]).getAbsoluteFile(),
                        new File(new File(args[x + 1]).getAbsolutePath()), args);

                        break;
                    }
                    catch (FileNotFoundException e) {
                        Output.initialError("A File Not Found Exception was generated upon"
                                            + "reading the parameter file \""
                                            + args[x + 1]
                                            + "\".\nHere it is:\n" + e);
                    }
                    catch (IOException e) {
                        Output.initialError("An IO Exception was generated upon reading the"
                                            + "parameter file \""
                                            + args[x + 1]
                                            + "\".\nHere it is:\n" + e);
                    }
                }

            if (parameters == null)
                Output.initialError("No parameter file was specified.");



                // 1. create the output

            store = !(parameters.getBoolean(new Parameter(P_NOSTORE), null,
                                            false));

            verbosity = parameters.getInt(new Parameter(P_VERBOSITY), null,
                                          0);

            if (verbosity < 0)
                Output.initialError("Verbosity should be an integer >= 0.\n",
                                    new Parameter(P_VERBOSITY));

            output = new Output(store, verbosity);

            output.setFlush(parameters.getBoolean(new Parameter(P_FLUSH),
                                                  null, false));


            // stdout is always log #0.  stderr is always log #1.
            // stderr accepts announcements, and both are fully verbose
            // by default.
            output.addLog(ec.util.Log.D_STDOUT, Output.V_VERBOSE, false);
            output.addLog(ec.util.Log.D_STDERR, Output.V_VERBOSE, true);




            // 2. set up thread values

            breedthreads = parameters.getInt(new Parameter(P_BREEDTHREADS),
                                             null, 1);

            if (breedthreads < 1)
                Output.initialError("Number of breeding threads should be an integer >0.",
                                    new Parameter(P_BREEDTHREADS));


            evalthreads = parameters.getInt(new Parameter(P_EVALTHREADS),
                                            null, 1);

            if (evalthreads < 1)
                Output.initialError("Number of eval threads should be an integer >0.",
                                    new Parameter(P_EVALTHREADS));



                // 3. create the Mersenne Twister random number generators,
                // one per thread

            random =
    new MersenneTwisterFast[breedthreads > evalthreads ? breedthreads : evalthreads];
            seeds =
                new int[breedthreads > evalthreads ? breedthreads : evalthreads];

            int time =
                (int) System.currentTimeMillis();           // safe because we're getting low-order bits

            for (x = 0; x < random.length; x++) {
                int seed = 1;
                String tmp_s =
                    parameters.getString(new Parameter(P_SEED).push("" + x),
                                         null);

                if (tmp_s == null)                          // uh oh
                 {
                    Output.initialError("Seed should be an integer.",
                                        new Parameter(P_SEED).push("" + x));

                }

                if (tmp_s.equalsIgnoreCase(V_SEED_TIME)) {
                    seed =
                        (int) System.currentTimeMillis();   // safe because we're getting low-order bits

                    if (seed == 0)
                        Output.initialError("Whoa! This Java version is returning 0 for System.currentTimeMillis(), which ain't right.  This means you can't use '"
                                            + V_SEED_TIME + "' as a seed ",
                                            new Parameter(P_SEED).push(""
                                                          + x));
                }
                else
                    seed =
                        parameters.getIntWithDefault(new Parameter(P_SEED).push(""
                            + x), null, 0);

                seeds[x] = seed;
            }

            for (x = 0; x < random.length; x++) {
                for (int y = x + 1; y < random.length; y++)
                    if (seeds[x] == seeds[y]) {
                        Output.initialError(P_SEED + "." + x + " ("
                                            + seeds[x] + ") and " + P_SEED
                                            + "." + y + " (" + seeds[y]
                                            + ") ought not be the same seed.");
                    }

                random[x] = new MersenneTwisterFast(seeds[x]);
            }

            // 4.  Start up the evolution

            // what evolution state to use?
            state =
                (EvolutionState) parameters.getInstanceForParameter(new Parameter(P_STATE),
                    null, EvolutionState.class);
            state.parameters = parameters;
            state.random = random;
            state.output = output;
            state.evalthreads = evalthreads;
            state.breedthreads = breedthreads;

            output.systemMessage(Version.message());

            // Retain from overwriting out.stat
            parameters.set(new Parameter("stat.file"), "pseudo-out.stat");

            state.setup(state, null);

            return state;
        }

        return null;
    }


}

